
#
# Perl Module for AceTag
#
# Cared for by Andy Williams <aw4@sanger.ac.uk>
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package AceTag;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );


my %fields = (
	      name => undef,
	      Values => undef
	      );

sub new {
    
    my $self = {
	'_permitted' => \%fields,
	%fields, 
    };

    $self->{'Values'} = [];
    bless $self;
    return $self;

}

sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || carp "Ok, $self is not an object of mine!";
    my $name = $AUTOLOAD;

    # don't propagate DESTROY messages...

    $name =~ /::DESTROY/ && return;

    $name =~ s/.*://; #get only the bit we want
    unless (exists $self->{'_permitted'}->{$name} ) {
	carp "In type $type, can't access $name - probably passed a wrong variable into Sequence  ";
    }
    if (@_) {
	return $self->{$name} = shift;
    } else {
	return $self->{$name}
    }
}

sub AddValue {
    
    my $self = shift;
    my $value = shift;

    push( @{ $self->{'Values'}}, $value);

}

sub EachValue {

    my $self = shift;
    my @array;

    foreach (@{ $self->{'Values'}}) {
	push(@array, $_);
    }
    return @array;
}



1;  # says use was ok
__END__

=head1 NAME

AceTag

=head1 DESCRIPTION

Description for B<AceTag>
AceTag is an internal module (internal to AceTagClient) that creates 
a data structure for returning acedb query results, hashed by Tag name. It
assume that the tag has already been created by AceTagSet.

=head2 AUTHOR

B<Andy Williams> Email aw4@sanger.ac.uk

=over

=head2 Example

Example to get you started......

                #!/usr/local/bin/perl

                use AceTag;

                # Add value to a Tag.
                # Tag name
                $value = "Em:Z81008 BLASTN 154.000000 885 949 71807 71871";
                $tag = "DNA_Homol";
           
                if (defined $self->{'Tag'}->{$tag}) {

		    # If Tag already exists

		    $t = $self->{'Tag'}->{$tag};
	            $t->AddValue($value);
		} else {
		    
		    # Create new tag.

	            $t = new AceTag;
		    $t->name($tag);
		    $t->AddValue($value);

		    # This command is from AceTagSet.pm but 
		    # is needed to create a new tag.

		    $self->AddTag($t);
		}
    

               

=head2 Functions

=item AddValue

B<AddValue($value)> Adds value to current Tag.


=item EachValue

B<EachValue()>. Returns all values for the current Tag.

=back

=head1 C<Further Information>

=for html
<a href="Aceclient.html">Aceclient</a>

=for html
<a href="AceTagSet.html">AceTagSet</a>

=for html
<a href="AceTagClient.html">AceTagClient</a>


