
#
# Perl Module for AceTagSet
#
# Cared for by Andy Williams <aw4@sanger.ac.uk>
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package AceTagSet;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use AceTag;
#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );


my %fields = (
	      Tag => undef
	      );

sub new {
    
    my $self = {
	'_permitted' => \%fields,
	%fields, 
    };

    $self->{'Tag'} = {};
    bless $self;
    return $self;

}

sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || carp "Ok, $self is not an object of mine!";
    my $name = $AUTOLOAD;

    # don't propagate DESTROY messages...

    $name =~ /::DESTROY/ && return;

    $name =~ s/.*://; #get only the bit we want
    unless (exists $self->{'_permitted'}->{$name} ) {
	carp "In type $type, can't access $name - probably passed a wrong variable into AceTagSet  ";
    }
    if (@_) {
	return $self->{$name} = shift;
    } else {
	return $self->{$name}
    }
}

sub AddTag {

    my $self = shift;
    my $tag = shift;
    my $name;


    $name = $tag->name();
#    print "Adding to hash $name to $self\n";
    $self->{'Tag'}->{$name} = $tag;
}

sub AddTagValuePair {

    my $self = shift;
    my $tag = shift;
    my $value = shift;
    my $t;

    if (defined $self->{'Tag'}->{$tag}) {
	# print "$tag Exists\n";
	$t = $self->{'Tag'}->{$tag};
	$t->AddValue($value);
    } else {
	# print "$tag is NEW\n";


	$t = new AceTag;
	$t->name($tag);
	$t->AddValue($value);
	$self->AddTag($t);
    }
}


sub EachTag {
    
    my $self = shift;
    my $name;
    my @array;

#    print "I've got a $self\n";



    foreach $name (keys %{ $self->{'Tag'}}) {
	# print "Adding $name to hash\n";
	push(@array, $self->{'Tag'}->{$name});
    }
    return @array;
}

sub GetTag {

    my $self = shift;
    my $tag = shift;
    my $name;

    $name = $self->{'Tag'}->{$tag};

    return $name;
}



1;  # says use was ok
__END__

=head1 NAME

AceTagSet

=head1 DESCRIPTION

Description for B<AceTagSet>

Adds tags and values to a hash array (hashed on tag).

=head2 AUTHOR

B<Andy Williams> Email aw4@sanger.ac.uk

=over

=head2 Example

Example to get you started...

                    #!/usr/local/bin/perl
    
                    use AceTagSet;

                    $result = new AceTagSet;
             
                    # Add a tag and value together
                    $a = "DNA_Homol";
                    $b = "Em:B14368 BLASTN 1910.000000 1189 808 2 383";
                    
                    $result->AddTagValuePair($a,$b);

                    # Get information from $result

                    foreach $tag ($result->EachTag()) {
    
			$name = $tag->name();
			print "$name\n";
			foreach $value ($tag->EachValue()) {
			    print "\t$value\n";
			}
			print "-------------------------------------------\n";
		    }

                    # Get data from a particular Tag.

                    $name = $result->GetTag("DNA_Homol");
                    
                    foreach $value ($name->EachValue()) {
    
			print "\t$value\n";
		    }


=head2 Functions

=item AddTag

B<AddTag($tag)>. Creates a new Tag called $tag.

=item AddTagValuePair

B<AddTagValuePair($tag, $value)>. Adds $value to tag $tag. If $tag
does not exist it uses B<AddTag($tag)> to create a new one.

=item EachTag

B<EachTag()>. Returns all tags in the current data structure.

=item GetTag

B<GetTag($tag)>. Returns a reference to the tag specified by $tag.

=back

=head1 C<Further Information>

=for html
<a href="Aceclient.html">Aceclient</a>

=for html
<a href="AceTagClient.html">AceTagClient</a>

=for html
<a href="AceTag.html">AceTag</a>


