package Bio::GSC::AA;
#----------------------------------------------------------------------
# General Documentation - Header
#----------------------------------------------------------------------

=head1 NAME Bio::GSC::AA

=head1 DESCRIPTION

Bio::GSC::AA is part of the Bio::GSC bundle of perl modules
developed at the Genome Sequencing Center at Washington University,
St. Louis, MO, USA.

Bio::GSC::AA contains functions to deal with polypeptide sequences.

=head1 API

The application programming interface is described below. Class methods
begin with an uppercase letter, instance methods with a lowercase
letter, and private methods with an underscore (these may not even be
documented).

=cut

#----------------------------------------------------------------------
# Directives
#----------------------------------------------------------------------
require 5.004;
require Exporter;
use strict;
use vars qw ($VERSION @ISA @EXPORT @EXPORT_OK);
$VERSION = "0.9";
@ISA = qw(Exporter);           
@EXPORT = qw();
@EXPORT_OK = qw(untranslateAA daltonAA);

#----------------------------------------------------------------------
# Class Variables
#----------------------------------------------------------------------

my %Untranslate = (
	'K' => 'AAR', 'N' => 'AAY', 'T' => 'ACN', 'R' => 'MGN',
	'S' => 'WSN', 'I' => 'ATH', 'M' => 'ATG', 'Q' => 'CAR',
	'H' => 'CAY', 'P' => 'CCN', 'L' => 'YTN', 'E' => 'GAR',
	'D' => 'GAY', 'A' => 'GCN', 'G' => 'GGN', 'V' => 'GTN',
	'Y' => 'TAY', 'C' => 'TGR', 'W' => 'TGG', 'F' => 'TTY',
	'*' => 'TRR', 'X' => 'NNN'
	);

my %Dalton = (
	'K' => 110, 'N' => 110, 'T' => 110, 'R' => 110,
	'S' => 110, 'I' => 110, 'M' => 110, 'Q' => 110,
	'H' => 110, 'P' => 110, 'L' => 110, 'E' => 110,
	'D' => 110, 'A' => 110, 'G' => 110, 'V' => 110,
	'Y' => 110, 'C' => 110, 'W' => 110, 'F' => 110,
	'*' => 110, 'X' => 110
);

#----------------------------------------------------------------------
# untranslateAA()
#----------------------------------------------------------------------

=head2 untranslateAA()

Description: Gives the degenerate nucleotide sequence corresponding to
an amino acid sequence. Single letter IUPAC codes are expected as
arguments. Degenerate IUPAC codes are returned.

 Usage: $nt = unTranslate($aa);

=cut

sub untranslateAA {
	my ($aa) = @_;
	my $nt = '';
	for(my $i=0;$i<length($aa);$i++) {
		$nt .= $Untranslate{substr($aa,$i,1)}
	}
	return $nt;
}

#----------------------------------------------------------------------
# daltonAA()
#----------------------------------------------------------------------

=head2 daltonAA()

Description: Calculates the molecular weight of a protein. Table needs
the true weight in daltons, not just 110.

 Usage: $weight = daltonAA($aa);

=cut

sub daltonAA {
	my ($aa) = @_;
}

#----------------------------------------------------------------------
# General Documentation - Footer
#----------------------------------------------------------------------

=head1 SEE ALSO

see other Bio::GSC modules.

=head1 AUTHOR

Ian Korf (ikorf@sapiens.wustl.edu)

=head1 REFERENCES

http://sapiens.wustl.edu/~ikorf/BioGSC

=head1 ACKNOWLEDGEMENTS

This software was developed at the Genome Sequencing Center at
Washington Univeristy, St. Louis, MO.

=head1 COPYRIGHT

Copyright (C) 1998 Washington University, St. Louis, MO, USA.
All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind.

=cut


1;

