package Bio::GSC::Constants;
#----------------------------------------------------------------------
# General Documentation - Header
#----------------------------------------------------------------------

=head1 NAME Bio::GSC::Constants

=head1 DESCRIPTION

Bio::GSC::Constants is part of the Bio::GSC bundle of perl modules
developed at the Genome Sequencing Center at Washington University,
St. Louis, MO, USA.

Bio::GSC::Constants define some global constants used by the Bio::GSC
modules. These will invade the namespace of any importing package.
Most of the constants are preceded by "gsc" so to avoid any collisions,
do not precede any of your names with "gsc". The only constants that
are not preceded by "gsc" are the boolean values TRUE and FALSE. Some
modules require constants so when using a subset of Bio::GSC (eg. use
Bio::GSC::Method), you should also use Bio::GSC::Constants.
Alternatively, you can just use Bio::GSC and get everything.

=cut

#----------------------------------------------------------------------
# Directives
#----------------------------------------------------------------------
require 5.004;
require Exporter;
use strict;
use vars qw ($VERSION @ISA @EXPORT);
$VERSION = "0.9";
@ISA = qw(Exporter);
@EXPORT = qw(
 TRUE
 FALSE

 gscRawInput
 gscFastaInput
 gscGenbankInput
 gscEmblInput

 gscIdentical
 gscSuperset
 gscSubset
 gscLong5
 gscLong3
 gscShort5
 gscShort3
 gscSkew5
 gscSkew3
 gscOverlap

 gscSimpleFeatures
 gscFlattenedFeatures
 gscIdenticalFeatures
 gscOverlappingFeatures
 gscIncompleteFeatures

 gscCoordinates
 gscGenomic
 gscScore

 gscSortGenomic
 gscSortScore
 gscSortP

 gscACEdumpMatchTable
 gscACEdumpFeature
 gscACEdumpDNA_Homol
 gscACEdumpPEP_Homol
 gscACEdumpSubsequence

 gscACEdumpBlastEst
 gscACEdumpBlastNRN
 gscACEdumpBlastNRP
 gscACEdumpGenscan
 gscACEdumpRepeatMasker

 gscDataCompression0
 gscDataCompression1
 gscDataCompression2
 gscDataCompression3
);

#----------------------------------------------------------------------
# General Use Constants
#----------------------------------------------------------------------

=head2 General Use Constants

Description: These are commonly-used constants:

 TRUE
 FALSE

=cut

use constant TRUE  => 1;
use constant FALSE => 0;

#----------------------------------------------------------------------
# File Type Sequeunce Initializers
#----------------------------------------------------------------------

=head2 File Type Sequeunce Initializers

Description: A Bio::GSC::Sequence may be instantiated from a number of
sources: raw, fasta, genbank, and embl.

 gscRawInput
 gscFastaInput
 gscGenbankInput
 gscEmblInput

=cut

use constant gscRawInput     => 11;
use constant gscFastaInput   => 12;
use constant gscGenbankInput => 13;   
use constant gscEmblInput    => 14;   


#----------------------------------------------------------------------
# Feature Comparison Operator Return Values
#----------------------------------------------------------------------

=head2 Feature Comparison Operator Return Values

Description: There are a number of feature comparison operators. They
return various values depending upon the function. See Bio::GSC::Feature
for more information.

 gscIdentical
 gscSuperset
 gscSubset
 gscLong5
 gscLong3
 gscShort5
 gscShort3
 gscSkew5
 gscSkew3
 gscOverlap

=cut

use constant gscIdentical => 20;
use constant gscSuperset  => 21;
use constant gscSubset    => 22;   
use constant gscLong5     => 23;   
use constant gscLong3     => 24;   
use constant gscShort5    => 25;   
use constant gscShort3    => 26;   
use constant gscSkew5     => 27;   
use constant gscSkew3     => 28;   
use constant gscOverlap   => 29;  


#----------------------------------------------------------------------
# Complex Feature Comparison Modes
#----------------------------------------------------------------------

=head2 Complex Feature Comparison Modes

Description: Complex features may be compared in a number of ways,
depending upon the nature of your criteria. See Bio::GSC::Feature
for more information.

 gscSimpleFeatures
 gscFlattenedFeatures
 gscIdenticalFeatures
 gscOverlappingFeatures
 gscIncompleteFeatures

=cut

use constant gscSimpleFeatures      => 41;   
use constant gscFlattenedFeatures   => 42;   
use constant gscIdenticalFeatures   => 43;   
use constant gscOverlappingFeatures => 44;
use constant gscIncompleteFeatures  => 45;


#----------------------------------------------------------------------
# Simple Feature Comparison Operators
#----------------------------------------------------------------------

=head2 Simple Feature Comparison Operators

Description: All features, simple or complex are compared using one of
the functions: cmpByCoor(), cmpByGenomic(), or cmpByScore(). cmpFeatures
requires specifying one of these operators. These tags are:

 gscCoordinates
 gscGenomic
 gscScore

=cut

use constant gscCoordinates => 61;
use constant gscGenomic     => 62;   
use constant gscScore       => 63;

#----------------------------------------------------------------------
# Feature Sorting Operators
#----------------------------------------------------------------------

=head2 Feature Sorting Operators

Description: There are a number of different ways to sort features.

 gscSortGenomic
 gscSortScore
 gscSortP

=cut

use constant gscSortGenomic => 81;   
use constant gscSortScore   => 82;
use constant gscSortP       => 83;

#----------------------------------------------------------------------
# Sequence Dumping Parameters
#----------------------------------------------------------------------

=head2 Sequence Dumping Parameters

Description: Currently these are ACEDB-specific. See
Bio::GSC::Sequence::acedbDump for more information.

 gscACEdumpMatchTable
 gscACEdumpFeature
 gscACEdumpDNA_Homol
 gscACEdumpPEP_Homol
 gscACEdumpSubsequence

 gscACEdumpBlastEst
 gscACEdumpBlastNRN
 gscACEdumpBlastNRP
 gscACEdumpGenscan
 gscACEdumpRepeatMasker

=cut

#----- general dumpers
use constant gscACEdumpFeature      => 101;
use constant gscACEdumpDNA_Homol    => 102;
use constant gscACEdumpPEP_Homol    => 103;
use constant gscACEdumpSubsequence  => 104;
use constant gscACEdumpMatchTable   => 105;

#----- GSC-specific dumpers
use constant gscACEdumpBlastEst     => 111;
use constant gscACEdumpBlastNRN     => 112;
use constant gscACEdumpBlastNRP     => 113;
use constant gscACEdumpGenscan      => 114;
use constant gscACEdumpRepeatMasker => 115;

#----------------------------------------------------------------------
# Data Compression Parameters
#----------------------------------------------------------------------

=head2 Data Compression Parameters

Description: parameters used for serializing.

 gscDataCompression0
 gscDataCompression1
 gscDataCompression2
 gscDataCompression3

=cut

use constant gscDataCompression0 => 121;
use constant gscDataCompression1 => 122;
use constant gscDataCompression2 => 123;
use constant gscDataCompression3 => 124;

#----------------------------------------------------------------------
# General Documentation - Footer
#----------------------------------------------------------------------

=head1 SEE ALSO

see other Bio::GSC modules.

=head1 AUTHOR

Ian Korf (ikorf@sapiens.wustl.edu)

=head1 REFERENCES

http://sapiens.wustl.edu/~ikorf/BioGSC

=head1 ACKNOWLEDGEMENTS

This software was developed at the Genome Sequencing Center at
Washington Univeristy, St. Louis, MO.

=head1 COPYRIGHT

Copyright (C) 1998 Washington University, St. Louis, MO, USA.
All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind.

=cut


1;

