package Bio::GSC::GAME;
#----------------------------------------------------------------------
# General Documentation - Header
#----------------------------------------------------------------------

=head1 NAME <put the module name here>

=head1 SYNOPSIS

<some samples of usage go here>

=head1 DESCRIPTION

Bio::GSC::GAME is part of the Bio::GSC bundle of perl modules
developed at the Genome Sequencing Center at Washington University,
St. Louis, MO, USA.

Bio::GSC::GAME is a Genome Analysis and Management Environment. It is
used to create flexible genomic sequence analysis pipelines.

=head1 ENVIRONMENT

<behavior of environment variables>

=head1 DEPENDENCIES


=head1 API

The application programming interface is described below. Class methods
begin with an uppercase letter, instance methods with a lowercase
letter, and private methods with an underscore (these may not even be
documented).

=cut

#----------------------------------------------------------------------
# Directives
#----------------------------------------------------------------------
require 5.004;
require Exporter;
use strict;
use vars qw ($VERSION @ISA @EXPORT @EXPORT_OK);
$VERSION = "0.9";
@ISA = qw(Exporter);           
@EXPORT = qw();
@EXPORT_OK = qw();

use Bio::GSC::Tool;

#----------------------------------------------------------------------
# Class Variables
#----------------------------------------------------------------------
my %Tool;
TOOLS: {
	my @Tool = Bio::GSC::Tool::getParserNames();
	foreach my $parser (@Tool) {$Tool{$parser} = 1}
}


#----------------------------------------------------------------------
# readGameplan
#----------------------------------------------------------------------

=head2 readGameplan()

Description:

 Usage:

=cut

sub readGameplan {
	my ($gameplan) = @_;
	my %plan;
	my %tool;
	open(GP, "<$gameplan") or die "ERROR: can't open $gameplan\n";

	my ($filemap, $title, $state, $tag);
	READ: {
		my $line = <GP>;
		last READ unless defined $line;

		#----- pitch the comments
		if ($line =~ /#/) {
			my $offset = index($line, "#");
			$line = substr($line, 0, $offset);
		}

		chomp($line);
		$state = $1 if $line =~ /^STATE\s+(\S+)/;
		if    ($line =~ /^TITLE\s+(\S+)/) {$title = $1}
		elsif ($line =~ /^FILEMAP\s+(\S+)/) {$filemap = $1}
		elsif ($line =~ /^([A-Z]+)\s+(\S+)/) {$plan{$state}{$1} = $2}
		elsif ($line =~ /^([A-Z]+)/) {$tag = $1}
		elsif ($line =~ /^\s+(.+)$/) {push(@{$plan{$state}{$tag}}, $1)}
		redo READ;
	}
	close GP;

	my $err = "ERROR: no such Tool exists in Bio::GSC::Tool";
	foreach my $state (keys %plan) {
		next unless defined $plan{$state}{'METHOD'};
		next if exists $Tool{$plan{$state}{'METHOD'}};
		die "$err <$plan{$state}{'METHOD'}>\n";
	}

	return ($title, $filemap, \%plan);
}


#----------------------------------------------------------------------
# compileGameplan
#----------------------------------------------------------------------

=head2 compileGameplan()

Description :

 Usage       :
 Arguments   :
 Returns     :

=cut

sub compileGameplan {
}


#----------------------------------------------------------------------
# runGameplan
#----------------------------------------------------------------------

=head2 runGameplan()

Description :

 Usage       :
 Arguments   :
 Returns     :

=cut

sub runGameplan {
}


#----------------------------------------------------------------------
# reportState
#----------------------------------------------------------------------

=head2 reportState()

Description :

 Usage       :
 Arguments   :
 Returns     :

=cut

sub reportState {
}


#----------------------------------------------------------------------
# General Documentation - Footer
#----------------------------------------------------------------------

=head1 SEE ALSO

see other Bio::GSC modules.

=head1 AUTHOR

Ian Korf (ikorf@sapiens.wustl.edu)

=head1 REFERENCES

http://sapiens.wustl.edu/~ikorf/BioGSC

=head1 ACKNOWLEDGEMENTS

This software was developed at the Genome Sequencing Center at
Washington Univeristy, St. Louis, MO.

=head1 COPYRIGHT

Copyright (C) 1998 Washington University, St. Louis, MO, USA.
All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind.

=cut


1;

