package Bio::GSC::Tool::Inverted;
#----------------------------------------------------------------------
# General Documentation - Header
#----------------------------------------------------------------------

=head1 NAME Bio::GSC::Tool::Inverted

=head1 DESCRIPTION

Bio::GSC::Tool::Inverted is part of the Bio::GSC bundle of perl
modules developed at the Genome Sequencing Center at Washington
University, St. Louis, MO, USA.

Bio::GSC::Tool::Inverted is a plug-in parser for an Inverted repeat
finder (Durbin and Mieg?).

=head1 DEPENDENCIES

 Bio::GSC::Feature

=cut

#----------------------------------------------------------------------
# Directives
#----------------------------------------------------------------------
require 5.004;
require Exporter;
use strict;
use vars qw ($VERSION @ISA @EXPORT @EXPORT_OK);
$VERSION = "0.9";
@ISA = qw(Exporter);           
@EXPORT = qw();
@EXPORT_OK = qw();

use Bio::GSC::Feature;

#----------------------------------------------------------------------
# Class Variables/Methods
#----------------------------------------------------------------------

#----------------------------------------------------------------------
# parse
#----------------------------------------------------------------------

sub parse {
	my ($this, $file, $filter) = @_;
	open(INV, "<$file") or die "ERROR: can't open $file\n";
	INVERTED: {
		my $line = <INV>;
		last INVERTED unless defined $line;
		redo INVERTED unless $line =~ /^Score/;

		my $inverted = new Bio::GSC::Feature;
		my ($score)   = $line =~ /Score\s+(\d+)/;
		my ($percent) = $line =~ /(\d+)%/;
		$line = <INV>;
		my ($begin, $end) = $line =~ /(\d+)\s+\S+\s+(\d+)/;
		$line = <INV>;
		$line = <INV>;
		my ($s_begin, $s_end) = $line =~ /(\d+)\s+\S+\s+(\d+)/;
		$inverted->setScore($score);
		$inverted->setPercent($percent);
		$inverted->setBegin($begin);
		$inverted->setEnd($end);
		$inverted->setSbjctBegin($s_begin);
		$inverted->setSbjctEnd($s_end);

		#----- check for filtering parameters
		$this->addFeature($inverted)
			if $inverted->queryFeature($filter);
		redo INVERTED;
	}
}


#----------------------------------------------------------------------
# General Documentation - Footer
#----------------------------------------------------------------------

=head1 SEE ALSO

 see other Bio::GSC modules.

=head1 AUTHOR

 Ian Korf (ikorf@sapiens.wustl.edu)

=head1 REFERENCES

 http://genome.wustl.edu/gsc/staff/ikorf/Bio::GSC

=head1 ACKNOWLEDGEMENTS

 This software was developed at the Genome Sequencing Center at
 Washington Univeristy, St. Louis, MO.

=head1 COPYRIGHT

 Copyright (C) 1998 Washington University, St. Louis, MO, USA.
 All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind.

=cut


1;

