package Bio::GSC::Tool::Quicktandem;
#----------------------------------------------------------------------
# General Documentation - Header
#----------------------------------------------------------------------

=head1 NAME Bio::GSC::Tool::Quicktandem

=head1 DESCRIPTION

Bio::GSC::Tool::Quicktandem is part of the Bio::GSC bundle of perl
modules developed at the Genome Sequencing Center at Washington
University, St. Louis, MO, USA.

Bio::GSC::Tool::Quicktandem is a plug-in parser for the Quicktandem
program (Durbin and Mieg?).

=head1 DEPENDENCIES

 Bio::GSC::Feature

=cut

#----------------------------------------------------------------------
# Directives
#----------------------------------------------------------------------
require 5.004;
require Exporter;
use strict;
use vars qw ($VERSION @ISA @EXPORT @EXPORT_OK);
$VERSION = "0.9";
@ISA = qw(Exporter);           
@EXPORT = qw();
@EXPORT_OK = qw();

use Bio::GSC::Feature;

#----------------------------------------------------------------------
# Class Variables/Methods
#----------------------------------------------------------------------

#----------------------------------------------------------------------
# parse
#----------------------------------------------------------------------

sub parse {
	my ($this, $file, $filter) = @_;
	open(TAN, "<$file") or die "ERROR: can't open $file\n";
	TANDEM: {
		my $line = <TAN>;
		last TANDEM unless defined $line;
		my ($foo, $len, $begin, $end, $gap) = split(/\s+/, $line);
		my $s_begin = $begin + $gap;
		my $s_end = $end + $gap;

		my $tandem = new Bio::GSC::Feature;
		$tandem->setBegin($begin);
		$tandem->setEnd($end);
		$tandem->setSbjctBegin($s_begin);
		$tandem->setSbjctEnd($s_end);

		#----- check for filtering parameters
		$this->addFeature($tandem)
			if $tandem->queryFeature($filter);
		redo TANDEM;
	}
}


#----------------------------------------------------------------------
# General Documentation - Footer
#----------------------------------------------------------------------

=head1 SEE ALSO

 see other Bio::GSC modules.

=head1 AUTHOR

 Ian Korf (ikorf@sapiens.wustl.edu)

=head1 REFERENCES

 http://genome.wustl.edu/gsc/staff/ikorf/Bio::GSC

=head1 ACKNOWLEDGEMENTS

 This software was developed at the Genome Sequencing Center at
 Washington Univeristy, St. Louis, MO.

=head1 COPYRIGHT

 Copyright (C) 1998 Washington University, St. Louis, MO, USA.
 All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind.

=cut


1;

