package Bio::GSC::Tool::RepeatMasker;
#----------------------------------------------------------------------
# General Documentation - Header
#----------------------------------------------------------------------

=head1 NAME Bio::GSC::Tool::RepeatMasker

=head1 DESCRIPTION

Bio::GSC::Tool::RepeatMasker is part of the Bio::GSC bundle of
perl modules developed at the Genome Sequencing Center at Washington
University, St. Louis, MO, USA.

Bio::GSC::Tool::RepeatMasker is a plug-in parser to Arian Smit's
repeat family finder.

=head1 DEPENDENCIES

 Bio::GSC::Feature

=cut

#----------------------------------------------------------------------
# Directives
#----------------------------------------------------------------------
require 5.004;
require Exporter;
use strict;
use vars qw ($VERSION @ISA @EXPORT @EXPORT_OK);
$VERSION = "0.9";
@ISA = qw(Exporter);           
@EXPORT = qw();
@EXPORT_OK = qw();

use Bio::GSC::Feature;

#----------------------------------------------------------------------
# Class Variables/Methods
#----------------------------------------------------------------------

#----------------------------------------------------------------------
# parse
#----------------------------------------------------------------------

sub parse {
	my ($this, $file, $filter) = @_;
	open(IN,"<$file") or die "ERROR: can't open $file\n";
	RM: while (<IN>) {
		last RM unless defined $_;
		next RM unless $_ =~ /^\s+\d/;
		my @field = split(/\s+/, $_);
		my ($foo, $family) = split(/\//,$field[11]);
		$family = $field[10] unless defined $family;

		my $repeat = new Bio::GSC::Feature;
		$repeat->setBegin($field[6]);
		$repeat->setEnd($field[7]);
		$repeat->setScore($field[1]);
		$repeat->setSbjctDef($family);

		#----- check for filtering
		next RM unless $repeat->queryFeature($filter);
		$this->addFeature($repeat);
	}
	close IN;
}


#----------------------------------------------------------------------
# General Documentation - Footer
#----------------------------------------------------------------------

=head1 SEE ALSO

 see other Bio::GSC modules.

=head1 AUTHOR

 Ian Korf (ikorf@sapiens.wustl.edu)

=head1 REFERENCES

 http://genome.wustl.edu/gsc/staff/ikorf/Bio::GSC

=head1 ACKNOWLEDGEMENTS

 This software was developed at the Genome Sequencing Center at
 Washington Univeristy, St. Louis, MO.

=head1 COPYRIGHT

 Copyright (C) 1998 Washington University, St. Louis, MO, USA.
 All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind.

=cut


1;

