package Bio::GSC::Tool::Xnu;
#----------------------------------------------------------------------
# General Documentation - Header
#----------------------------------------------------------------------

=head1 NAME Bio::GSC::Tool::Xnu

=head1 DESCRIPTION

Bio::GSC::Tool::Xnu is part of the Bio::GSC bundle of perl modules
developed at the Genome Sequencing Center at Washington University,
St. Louis, MO, USA.

Bio::GSC::Tool::Xnu is a plug-in parser for the Xnu repeat finder. It
is designed for filtering protein not nucleotide sequence (PAM120 is
used internally). You must use options -r -. for this parser.

=head1 DEPENDENCIES

 Bio::GSC::Feature
 Bio::GSC::Utilities

=cut

#----------------------------------------------------------------------
# Directives
#----------------------------------------------------------------------
require 5.004;
require Exporter;
use strict;
use vars qw ($VERSION @ISA @EXPORT @EXPORT_OK);
$VERSION = "0.9";
@ISA = qw(Exporter);           
@EXPORT = qw();
@EXPORT_OK = qw();

use Bio::GSC::Feature;
use Bio::GSC::Utilities qw(getFasta);

#----------------------------------------------------------------------
# Class Variables/Methods
#----------------------------------------------------------------------


#----------------------------------------------------------------------
# parse
#----------------------------------------------------------------------

sub parse {
	my ($this, $file, $filter) = @_;
	my ($def, $seq) = getFasta($file);
	my ($i, $start, $stop);
	XNU: for($i=0;$i<length($seq)-1;$i++) {
		$start = index($seq,'.',$i);
		last XNU if $start == -1;
		$i = $start;
		while (substr($seq, $i, 1) eq '.') {
			$stop = $i;
			$i++;
		}
		# make a new feature object
		my $repeat = new Bio::GSC::Feature;
		$repeat->setBegin($start);
		$repeat->setEnd($stop);

		#----- check for filtering parameters
		next XNU unless $repeat->queryFeature($filter);
		$this->addFeature($repeat);
	}
}


#----------------------------------------------------------------------
# General Documentation - Footer
#----------------------------------------------------------------------

=head1 SEE ALSO

 see other Bio::GSC modules.

=head1 AUTHOR

 Ian Korf (ikorf@sapiens.wustl.edu)

=head1 REFERENCES

 http://genome.wustl.edu/gsc/staff/ikorf/Bio::GSC

=head1 ACKNOWLEDGEMENTS

 This software was developed at the Genome Sequencing Center at
 Washington Univeristy, St. Louis, MO.

=head1 COPYRIGHT

 Copyright (C) 1998 Washington University, St. Louis, MO, USA.
 All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind.

=cut


1;

