#
# Perl Module for HomolGeneFeature
#
# Cared for by Tim Hubbard <th@sanger.ac.uk>
#
# $Header: /nfs/disk103/th/dir/progs/PerlModules/RCS/HomolGeneFeature.pm,v 1.3 1998/07/14 02:25:33 th Exp $
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package HomolGeneFeature;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter','GeneFeature' );

#
# Method to parse a line from an MSP file from MSPcrunch -d output
# 


# need these for the permitted thing..

my %fields = (
	      seqname => undef, 
	      source  => undef,
	      feature  => undef,
	      start => undef,
	      end => undef,
	      score => undef,
	      strand => undef,
	      frame => undef,
	      group => undef,
	      pairs => undef,
	      member => undef,
	      start2 => undef,
	      percentid => undef,
	      end2 => undef,
);

sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    
    my $self;
    $self = new GeneFeature; # make our base class
    
    # watch it - we have to keep the fields in the above
    # in sync with genefeature. Ugh!

    $self->{'_permitted'} = \%fields;

    bless $self, $class;
    return $self;
}

# for reading a line for an MSPcrunch -d format
sub new_from_msp {
    my $string = shift;
    my $source = shift;
    my $function = shift;
    my ($self,$group);
    my @array;

    $self = new HomolGeneFeature;
# predefined
    $self->source($source);
    $self->feature('similarity');
# undefined
    $self->strand(0);
    $self->frame(0);

    chomp($string);

# remove leading spaces
    $string=~s/^\s+//;

    @array = split(/\s+/,$string);

# load in order
# raw score
    $self->score(shift(@array));
# ignore %id
    $self->percentid(shift(@array));
    $self->start(shift(@array));
    $self->end(shift(@array));
    $self->seqname(shift(@array));
    $self->start2(shift(@array));
    $self->end2(shift(@array));

# allow special rule to be used to parse name
    $group=shift(@array);
#    print "|$group|\n";
    if($function){
	$group=&$function($group);
    }
#    print "|$group|\n";
#    exit 0;

    $self->group($group);

    return $self;
}

#
# Method to return a formatted output of a GeneFeature object to a filehandle.
# Uses dump_string
#

sub dump {
    my $self = shift;
    my $file = shift;
    my $tab = shift;
    
    if( !defined $file ){
	$file = \*STDOUT;
    }

    print $file $self->dump_string($tab)."\n";
}

#
# Method to return a formatted output of a GeneFeature object to a string
#

sub dump_string {
    my $self = shift;
    my $tab = shift;

    if($tab){
	sprintf("%s\t%s\t%s\t%d\t%d\t%f\t%d\t%d\t%d\t%s",
		$self->seqname(),
		$self->source(),
		$self->feature(),
		$self->start(),
		$self->end(),
		$self->score(),
		$self->start2(),
		$self->end2(),
		$self->percentid(),
		$self->group());
    }else{
	sprintf("%s %s %s %d %d %f %d %d %d %s",
		$self->seqname(),
		$self->source(),
		$self->feature(),
		$self->start(),
		$self->end(),
		$self->score(),
		$self->start2(),
		$self->end2(),
		$self->percentid(),
		$self->group());
    }
}


1;  # says use was ok
__END__

=head1 NAME

HomolGeneFeature

=head1 DESCRIPTION

Description for B<HomolGeneFeature>

=head1 AUTHOR

B<Tim Hubbard> Email th@sanger.ac.uk

=over
