
#
# Perl Module for PfamDatabase.pm
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package PfamDatabase;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use PfamModel;


#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );





my %fields = ( 
	      release  => undef,
	      domain => [],
	      );

sub getstd_release_subset {
    my $self = shift;
    my @wanted = @_;
    my $name;
    my $rel_dir;
    my ($wstring,$mod,$num);

    $wstring = join ':', @wanted;
    


    $rel_dir = $PfamModel::release_dir;



    if( !open(HMM,"$rel_dir/HMMs") ) {
	carp("Could not open the file $rel_dir/HMMs");
	return 0;
    }

    while(<HMM>) {
	!/^(\S+)\s/ && next;
	
	$name = $1;
	if( !($wstring =~ /$name/) ) { next; }


	$mod = new PfamModel;

	$mod->name($name);
	$mod->getstd_model();

	push(@{$self->domain()},$mod);
	$num++;
	
    }

    return $num;

}

sub set_global_threshold {
    my $self = shift;
    my $thresh = shift;
    my $pfm;

    foreach $pfm ( @{$self->{domain}} ) {
	$pfm->thresh($thresh);
    }
}

sub set_global_program {
    my $self = shift;
    my $prog = shift;
    my $pfm;
    

    foreach $pfm ( @{$self->{domain}} ) {
	$pfm->hmmprog($prog);
    }
}

sub get_current_families {
    my $current_dir;
    my (@arr,$trash);

    $current_dir = $PfamModel::current_dir;

    if( !opendir(DIR,$current_dir) ) {
	carp("Could not open current_dir Really bad!");
	die;
    }

    # yuk - getting rid of . and ..
    # very bad!

    $trash = readdir DIR;
    $trash = readdir DIR;

    @arr = readdir DIR;

    closedir(DIR);

    return @arr;
}

sub flatfile_release {
    my $self = shift;
    my $hmmsfile = shift;
    my $directory = shift;
    my $count = 0;
    my ($rnd,$name,$prog,$thresh,$pm);

    if( !defined $directory ) {
	$directory= ".";
    }

    if( !open(T,$hmmsfile) ) {
	carp("Could not open $hmmsfile as PfamDatabase file $!");
	return 0;
    }

    
    while( <T> ) {
	chop;
	$rnd = 0;
	if( !/(\S+)\s+(\S+)\s+(\d+)/ ) {
	    carp("In file $hmmsfile, got a dodgy line [$_]");
	}

	$name = $1;
	$prog = $2;
	$thresh = $3;

	if( $prog =~ s/\-r$// ) {
	    $rnd = $1;
	}

	$pm = new PfamModel;

	$pm->name($name);
	$pm->hmmprog($prog);
	$pm->thresh($thresh);
	$pm->hmmfile("$directory/$name.HMM");
	
	if( $rnd == 1) {
	    $pm->rndfile("$directory/$name.random");
	}

	push(@{$self->domain()},$pm);

	$count++;
    }

    close(T);

    return $count;
}

sub get_family_list {
    my @out;
    my $rel_dir;
    my $name;

    $rel_dir = $PfamModel::release_dir;



    if( !open(HMM,"$rel_dir/HMMs") ) {
	carp("Could not open the file $rel_dir/HMMs");
	return 0;
    }

    while(<HMM>) {
	!/^(\S+)\s/ && next;

	$name = $1;
	push(@out,$name);
	
    }
    return @out;
}


sub compare_overlap_to_current {
    my $one = shift;
    my $report = shift;
    my @families;
    my @arry;
    my (%hash,$seq,$family,$model,$startstop,@startstop,$start,$stop);

    # use main ALIGN-ment.

    # build a hash of one on sequence names:

    foreach $seq ( $one->align()->eachSequence() ) {
	$hash{$seq->name()} .= sprintf("%d-%d:",$seq->start(),$seq->end());
    }

    #
    # get current families to loop over
    #

    @families = get_current_families();

    #
    # foreach family, foreach sequence, find if it is in other sequence.
    #

    foreach $family ( @families ) {
	$model = new PfamModel;
	$model->get_current_model($family);

	foreach $seq ( $model->align()->eachSequence() ) {
	    if( $hash{$seq->name()} ) {
		$_ = $hash{$seq->name()};
		chop; # trailing :
		@startstop = split(/:/);
		foreach $startstop ( @startstop) {
		    ($start,$stop) = split(/-/,$startstop);
		    if( ($start >= $seq->start() && $stop <= $seq->end()) || 
		        ($start <= $seq->end() && $stop >= $seq->end()) ||  
		        ($start <= $seq->start && $stop >= $seq->start()) ) {
			push(@arry,sprintf("%s:%s/%d-%d:%s/%d-%d|",$seq->name(),$one->name(),$start,$stop,$model->name,$seq->start(),$seq->end()));
		    }
		}
	    }
	}
	if( $report ) {
	    print $report sprintf("Done Model %s",$model->name());
	}

	$model = undef; # free Model
    } # end of for each family

    return @arry;
}
    
sub getstd_release {
    my $self = shift;
    my $num = 0;
    my ($name,$mod);
    my $rel_dir;

    $rel_dir = $PfamModel::release_dir;



    if( !open(HMM,"$rel_dir/HMMs") ) {
	carp("Could not open the file $rel_dir/HMMs");
	return 0;
    }

    while(<HMM>) {
	!/^(\S+)\s/ && next;

	$name = $1;

	$mod = new PfamModel;

	$mod->name($name);
	$mod->getstd_model();

	push(@{$self->domain()},$mod);
	$num++;
	
    }

    return $num;




    $mod = new PfamModel;
    $mod->name("rrm");
    $mod->getstd_model();
    push(@{$self->domain()},$mod);

    return $self;
}

sub show {
    my $self = shift;
    my $file = shift;
    my $pm;


    print "Entring\n";

    foreach $pm ( @{$self->{domain}} ) {
	$pm->show($file);
    }
}
    
    
sub new {
    my $ref = shift;
    my $class = ref($ref) || $ref;
    
    my $self = {
	_permitted => \%fields,
	%fields, };

    $self->{domain} = [];

    bless $self, $class;
    return $self;
}



sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || warn "Ok, $self is not an object of mine!";
    my $name = $AUTOLOAD;
    $name =~ s/.*://; #get only the bit we want
    unless (exists $self->{_permitted}->{$name} ) {
	warn "In type $type, can't access $name  ";
    }
    if (@_) {
	return $self->{$name} = shift;
    } else {
	return $self->{$name}
    }
}

    
sub DESTROY {  }

END { }       # module clean-up code here (global destructor)


1;            # modules must return true


__END__

=head1 NAME

PfamDatabase

=head1 DESCRIPTION

Description for B<PfamDatabase>

=head1 AUTHOR

B<Ewan Birney> Email birney@sanger.ac.uk

=over



