
#
# Perl Module for ProgramMinder
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#

package ProgramMinder;

use vars qw($AUTOLOAD @ISA @EXPORT_OK @EXPORT);
use Exporter;
use Carp;
use strict;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT = qw( NeedProgram );

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );


sub NeedProgram {
    my @programs = @_;
    my $program;
    my $out = 1;
    my @baduns;


    foreach $program ( @programs ) {
	open(WHI,"which $program |") || confess "Could not open which!";
	while( <WHI> ) {
	    
	    if( /[Cc]ommand not found/ || /^no $program/) {
		push(@baduns,$program);
		$out =0;
	    }
	}
    }
    if( $out == 1 ) {
	return 1;
    }

    print STDERR ("Program Minder unable to find some needed programs\n");

    foreach $program (@baduns) {
	print STDERR "\t$program\n";
    }

    confess("ProgramMinder exiting now");

    return 0;

}



1;  # says use was ok

__END__

=head1 NAME

ProgramMinder

=head1 DESCRIPTION

ProgramMinder gives a way to check that programs exsist
before using them later in the script. It is sort of the equivalent
to use <Module> for executables.

=head1 AUTHOR

B<Ewan Birney> Email birney@sanger.ac.uk

=head1 Example

	use ProgramMinder;
	
	NeedProgram("efetch","blastp","sw"); # dies dramatically if not

	#
	# Do normal stuff here...
	#

=head1 Methods

=over

=item NeedProgram

Calling NeedProgram(program names) (in a list) means that each Program is currently
found using a which system call. If there is no program by that name, it will die
dramatically, telling you all the programs it can't fine and where it was called
from.

NeedProgram is exported directly into your namespace when you go C<use ProgramMinder;>
you don't need any 'new' etc call

=back


