
#
# Actual this is only documentation!
#
#


__END__

=head1 NAME

Introduction/Style Guide for good Perl Modules programming.

=head1 DESCRIPTION

This document is meant to give you some hints on how
to provide a Perl Module into this archive, what is
expected of you as an author, and what users can expect
from the archive.

=head1 Getting Started (user)

To use one of the perl Modules you should

a) pick up from the ftp site the Module of interested. It
should tell you in the documentation what other modules
you need to pick up as well.

With the ftp site, you can use the auto-tar features of
our ftp server to provide you with whole directories (or indeed, the
entire package).

b) place them in the Modules directory of your Perl installation,
or, put them into the a directory which the environment variable
"PERL5LIB" is set to, or use the perl command

    BEGIN  { 
	unshift(@INC,"/nfs/MyDirectory/Perl/");
    }

or where you put the modules

c) go

    
    use ModuleName;

and follow the examples.


=head1 Introduction to the Archive

The Perl Modules archive was set up originally as an internal
archive at the Sanger centre for people to provide and use 
Perl Modules. This has slowly grown to something where other
people, external to the site want to use the modules and/or
contribute to them.

It is important to recognise that the Perl Modules are provided
by individuals and reflect individual programming needs and style.
There is B<no> sweeping architecture underlying the perl modules
(but that one uses perl and the package directive): Although we expect
to care for the Modules that we wrote, there is no guarentee that
somehow these modules will be developed/bug fixed/ etc. However we
do guarentee that you can use the source code as much as you like,
which means B<you> can bug-fix, modify etc...


=head2 Why use perl modules?

The choice of Perl is easy - it is by far the most portable and
widely known scripting languaging in bioinformatics. Python is
very neat, but ultimately nearly every bioinformatics site will
have Perl installed by default.

Although programmers have complained quite loudly about Perl's short
comings for 'programming-in-the-large' most of the complaints are
aesthetic (it doesn't look nice - this can be an understatement), and from the point of view of
providing a clean user interface, the user probably wont know (nor
care...) that s/he is using some incredibly tortuous memory system. Hackers
usually B<enjoy> getting their heads around the wacky internals of a module.

The package directive of Perl is ideal to provide your own namespace
critical for writing modular code. Besides that we enforce no
particular style. We have suggestions for some things, in particular
documentation, but you don't need to follow it.

Finally the documentation, in pod in the modules is precisely what
most modules need: up-to-date Documentation.  Documentation is second
only to encapsulation as a necessary step for a decent module.

=head2 Is this the only bio-perl archive

No. There is the very well made bio:: heirarchy provided by a variety
of authors. They take a much more considered view of the objects required
than this collection, and the package is a more of a fully working set
of biological objects. At the sanger centre we do use some of their modules,
and it may well be that they have modules which are better suited to some tasks.

Check out the web site at http://genome-www.stanford.edu/perlOOP/bioperl/
or http://www.techfak.uni-bielefeld.de/bcd/Perl/Bio/

=for html
These are hotlinked here <a href="http://genome-www.stanford.edu/perlOOP/bioperl/">
stanford </a> and <a href="http://www.techfak.uni-bielefeld.de/bcd/Perl/Bio/"> Germany </a>


=head1 Contributing

=head2 What can I contribute?

Anything that uses the package directive.

Ideally of course it should be concerned with bioinformatics, and hopefully
does not overlap badly in terms of functionality with something existing. If
you can tack something on to an existing module, then please try to do that.


=head2 What should what I contribute look like?

Ideally it should look like the Modules in the archive. In terms of Perl 
structure, it should definitely start

    package PackageName;


and end with


    1;

which indicates a valid package.

The next most important thing is that the documentation should be easily
read from pod, the principle test being html.

The documentation often starts after __END__ ie.

    1; # end of package
    __END__ # perl stop's parsing

    \=head1 NAME 
    
    PackageName

Lets stress this more. B<Documentation is critical> to the success
of your Module. We'll flesh this out later, but always document: document
things verbosely if in doubt. And provide lots of examples.

A variety of things are now nice if people want to make the package 
more useful

=over

=item use strict

If you use strict you (and other people) catch more bugs at compile time

=item use Exporter

and inhierit from it from people to use your modules as base classes

=item constants at the top

Group all your constants together at the top so they can be easily
modified

=item document if in doubt

Just to remind you ;)

=back

=head2 How do I contribute?

At the moment, we don't have a good set up for contributions. Get in
touch with either Ewan Birney 

=for html
<a href="mailto:birney@sanger.ac.uk"> 

birney@sanger.ac.uk

=for html
</a>

or Simon Dear

=for html
<a href="mailto:sd@sanger.ac.uk"> 

sd@sanger.ac.uk 

=for html
</a> 

for more information


=head1 Documentation style - One man's view.

The only must in documentation is that it should be useful to two types
of people. B<users> who want to treat the module as a black box and
B<developers> who want to expand/modify/bug-fix the modules.

The most important thing in the documentation is to provide a simple
example (as real life as possible) and then documentation for each
function provided. Otherwise more documentation is usually better than
less.

I put all the documentation together at the end (not in the code) 
because I think documentation is more critical for people who
want to understand the module without understanding the code.
Documentation has to read for naive users. Coders usually don't
need that much documentation (or can find it if they need it),
and need documentation style of their own (eg, beware, crazy use
of $\ here), which is no use to a 'user' of the module.


=head1 Documentation style - bioperl's view

bioperl places documentation inside the source, bringing the documentation
of the function closer to the actual function body. They have a standard
which is based around the following text:

 Title     : 
           : 
 Usage     : 
           :
 Purpose   : 
           :
 Example   : 
           :
 Returns   : 
           :
 Argument  : 
           :
 Throws    : 
           :
 Access    : Public | Private | Protected
           :
 Status    : Exported | Experimental | Virtual | Stub
           :
 Comments  : 
           :

  See Also   :

=head1 Object Perl primers

There are a variety of Perl OOP primers out there. First to look
at is the one bundled with the Perl documentation (ie, perloot)

=for html
There is a local copy at the sanger centre as <a href="http://www.sanger.ac.uk/Software/PerlModule/perl5.004/perltoot.html"> Perl Object tutorial </a>

Around the Perl world quite alot Object 'primers' have been developed.
One such primer is http://world.std.com/~swmcd/steven/perl/

=for html 
hotlinked <a href="http://world.std.com/~swmcd/steven/perl/"> here </a>















