#
# Perl Module for Aceclient
#
# Cared for by Andy Williams <aw4@sanger.ac.uk>
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package Tace;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
#use strict;
use AceTagSet;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

my %fields = (
	      program  => undef,
	      directory => undef,
	      );
my %Machines = ("Irix", "IRIXbin",  
		"Linux", "LINUXbin",
		"OSF1", "OSFbin",    
		"Solaris", "SOLbin",    
		"SunOS", "SUNbin"#
		);


my $os = `ostype`;
chop $os;

sub new {
    
    my $self = {
	'_permitted' => \%fields,
	%fields, 
    };
    bless $self;
    return $self;

}

sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || carp "Ok, $self is not an object of mine!";
    my $name = $AUTOLOAD;

    # don't propagate DESTROY messages...

    $name =~ /::DESTROY/ && return;

    $name =~ s/.*://; #get only the bit we want
    unless (exists $self->{'_permitted'}->{$name} ) {
	carp "In type $type, can't access $name - probably passed a wrong variable into Sequence  ";
    }
    if (@_) {
	return $self->{$name} = shift;
    } else {
	return $self->{$name}
    }
}


sub connect {
    my $self = shift;
    my $program = shift;
    my $directory = shift;
    my $query = "quit";

    if (! defined $program) {
	$program = "tace";
    }
    if ($program eq "pace") {
	$program = "pace -t";
    }
    if (! defined $directory) {
	if ($program ne "pace -t") {
	    $directory = "/nfs/humace/humpub/humace";
	}
    }
    
    if (open(CLIENT, "echo '$query' | $program $directory |")) {
	while (<CLIENT>) {
	    if (/cannot\s+establish\s+connection/) {
		close CLIENT;
		return 0;
	    } else {
		close CLIENT;
		$self->program($program);
		$self->directory($directory);
		
	        return 1;
	    }
	}
    } else {
	return 0;
    }
}

sub query {
    my $self = shift;
    my $query = shift;
    my ($l, $f);
    my @result;
    my %Results;

    # Query must be seperated by ;
    
    my $program = $self->program();
    my $directory = $self->directory();

    if (! defined $query) {
	return 0;
    }
    $query =~ s/\&/\n/g;
    
    open(CLIENT, "echo '$query' | $program $directory |") or return 0;

    my $result = new AceTagSet;

    while (<CLIENT>) {
	if (! /(\/\/)|(^acedb)/) {
	   ($a, $b) = split (/\s+/, $_, 2);
	   $a =~ s/\s+//g;
	   $b =~ s/\"//g;
	   $b =~ s/\s\:\s/\s/g;
	   
	   chop($b); 
	   $result->AddTagValuePair($a,$b);

	} else {
	    $result->AddTagValuePair("LOG",$_);
	}
	
    }

    return $result; # give back a AceTagSet
}

sub getDNA {

    my $self = shift;
    my $seq = shift;
    my @result;
    my $query = "find sequence $seq & dna & quit";
    my $program = $self->program();
    my $directory = $self->directory();

    if (! defined $seq) {
	return 0;
    }
    $query =~ s/\&/\n/g;
    
    open(CLIENT, "echo '$query' | $program $directory |") or return 0;

   
    while (<CLIENT>) {
	if (! /(\/\/)|(^acedb)|(^\s+)/) {
	    push @result, $_;
	}
    }
    close CLIENT;
    return @result;    
}

sub DNA_fasta {

    my $self = shift;
    my $seq  = shift;
    my $name = shift;
    my $program = $self->program();
    my $directory = $self->directory();
    
    if (! defined $seq) {
	return 0;
    }
   
    if (! defined $name) {
	my $work = `pwd`;
	chop $work;
	$name = "$work/$seq.seq";
	print "Name = $name\n";
    }
    
    my $query = "find sequence $seq & dna -f $name & quit";
    $query =~ s/\&/\n/g;
    
    open(CLIENT, "echo '$query' | $program $directory |") or return 0;

   
    while (<CLIENT>) {
    }
    close CLIENT;
    return 1;
}






1;  # says use was ok
__END__

=head1 NAME

Tace

=head1 DESCRIPTION

B<Tace.pm> Allows access to an ACEDB database. Allows
quering of any ACEDB datbase. 

=head2 AUTHOR

B<Andy Williams> Email aw4@sanger.ac.uk

=over

=head3 Example1

A simple example to get us started.

		#!/usr/local/bin/perl

		use Tace;

		# Create a new connection to ACEDB

		$test = new Tace;

		$out = $test->connect("tace", "/nfs/humace/humpub/humace");

		# Show everything about a clone.

		$command = "find genome_sequence cN117F11 & show -a & quit";

		$results = $test->query("$command");

                foreach $tag ($result->EachTag()) {
    
		    $name = $tag->name();
		    print "$name\n";
		    foreach $value ($tag->EachValue()) {
			print "\t$value\n";
		    }
		}

                # Find a specific tag.
 
                $tag = $results->GetTag("DNA_Homol");
                foreach $value ($tag->EachValue()) {
                    print "\t$value\n";
                }

=head1 Creating a new connection

=over 5

=item C<new>

C<Tace::new>

Creates connection.

=item C<connect>

B<connect("$program", "$directory")>.Connects to a specified server and 
port number. Returns 0 if connection fails. If $program and/or
$directory not specified defaults (currently tace and /nfs/humace/humpub/humace) are
used. You can also connect to the pace database B<connect("pace")>.

=over5

=head1 C<Functions>

=item C<query>

B<query("$query")>. Queries the current ACEDB and returns
results, without comments, in a hash array - hashed on the tags. 
$query should in the form B<find sequence & show -a> ie seperated
by '&'.

=item C<getDNA>

B<getDNA("$sequence")>. Queries ACEDB and returns the DNA
output to a variable.

=item C<DNA_fasta>

B<DNA_fasta("$sequence")> or B<DNA_fasta("$sequence","$file")>.
As for B<getDNA> but write output to a file. If no $file is
specified then the output is written to (in the current working
directory) $sequence.seq. If $file is specified it must be the
full path to the output file.


=back
