#!/usr/local/bin/perl

package humConf;

# Can't say "use strict" since symbolic refs are used

# Could change user in future
my $humpub = ( getpwnam('humpub') )[7];
my $badger = ( getpwnam('badger') )[7];
my $ftp    = ( getpwnam('ftp')    )[7];

# Edit this hash to change config
my %humConf = (
	       'HUMAN_SEQ_FTP_DIR' => "$ftp/pub/human/sequences",
	       'HUMPUB_ROOT' => $humpub,
	       'HUMACE_DIR' => '/nfs/humace/humpub/humace',
	       'HUMACE_RO_DIR' => '/nfs/humace/humpub/humace-live-ro',
	       'HUMACE_QUEUE' => "$humpub/humace/queue",
	       'HUMACESERVER_HOST' => 'humsrv1',
	       'HUMACESERVER_PORT' => 210000,
	       'HUMGIFACESERVER_PORT' => 310000,
	       'HUMACE_CLIENT_TIMEOUT' => 1200,
	       'CONFIG_DEFAULT' => "$humpub/scripts/haceprep.cfg",
	       'ACESERVER_1_PORT' => 100100,
	       'ACESERVER_6_PORT' => 100103,
	       'ACESERVER_20_PORT' => 100101,
	       'ACESERVER_22_PORT' => 100102,
	       'ACESERVER_X_PORT' => 100104,
	       'BADGER_ROOT' => $badger,
	       'PACE_DIR' => "$badger/pace",
	       'PACE_QUEUE' => "$badger/pace/queue",
	       'GF_DATA' => "$humpub/solovyev/fgenes_run"
	       );

sub import {
    my ($callpack) = caller(0); # Name of the calling package
    my $pack = shift; # Need to move package off @_

    # Get list of variables supplied, or else
    # all of humConf:
    my @vars = @_ ? @_ : keys( %humConf );
    return unless @vars;

    # Predeclare global variables in calling package
    eval "package $callpack; use vars qw("
         . join(' ', map { '$'.$_ } @vars) . ")";
    die $@ if $@;

    foreach (@vars) {
	if ( defined $humConf{ $_ } ) {
	    # Exporter does a similar job to the following
	    # statement, but for function names, not
	    # scalar variables:
	    *{"${callpack}::$_"} = \$humConf{ $_ };
	} else {
	    die "Error: humConf: $_ not known\n";
	}
    }
}

sub list {

    eval {
	open LIST_HUM_VARS, ">-"; # open to STDOUT
	print LIST_HUM_VARS "\n";
	foreach $key ( sort keys( %humConf ) ) {
	    local $value = $humConf{ $key };
	    write LIST_HUM_VARS;
	}
	print LIST_HUM_VARS "\n";
	close LIST_HUM_VARS;

format LIST_HUM_VARS = 
 @>>>>>>>>>>>>>>>>>>>>>>>>>> = ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $key,                         $value
~                               ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
                                $value
~                               ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
                                $value
~                               ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
                                $value
~                               ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<...
                                $value
.
}; warn $@ if $@;
}

return 1;


__END__

=head1 NAME

humConf - imports global variables used by human sequence analysis

=head1 SYNOPSIS

    use humConf;
    use humConf qw( HUMACESERVER_HOST HUMACESERVER_PORT );

    &humConf::list();

=head1 DESCRIPTION

humConf is based upon ideas from the standard perl
Env environment module.

It imports and sets a number of standard global
variables into the calling package, which are used
in many scripts in the human sequence analysis
system.  The variables are first decalared using
"use vars", so that it can be used when "use
strict" is in use in the calling script.  Without
arguments all the standard variables are set, and
with a list, only those variables whose names are
provided are set.  The module will die if a
variable which doesn't appear in its C<%humConf>
hash is asked to be set.

Edit C<%humConf> to add or alter variables.

All the variables are in capitalis, so that they
resemble environment variables.

The C<humConf::list()> function prints a nicely
fomatted list of the variables known to humConf.

=head1 AUTHOR

B<James Gilbert> email jgrg@sanger.ac.uk

