#
# Perl Module for humpub
#
# Cared for by Tim Hubbard <th@sanger.ac.uk>
#
# $Header: /nfs/disk100/humpub/modules.devel/RCS/humpub.pm,v 1.32 1998/11/24 18:09:06 humpub Exp $
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package humpub;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use File::Copy;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw(getdate);

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

# set global humace variables into main
sub humace_conf{
# glob cost is too high due to defunct processes
#    $main::humpub_home=glob "~humpub";
    $main::humpub_home="/nfs/disk100/humpub";
    $main::humace_home="/nfs/humace/humpub/humace";
    $main::humace_queue="$main::humpub_home/humace/queue";
    $main::gff_dump_dir="$main::humpub_home/th/gffdump";
    $main::delete_dump_dir="$main::humace_home/database/deleted";
    $main::humaceserver_host='humsrv1';
    $main::humaceserver_port=210000;
    $main::humacegserver_port=310000;
    $main::humaceclient_timeout=1200;
    $main::config_default="$main::humpub_home/scripts/haceprep.cfg";
    $main::aceserver_port{'humace'}=$main::humacegserver_port;
    $main::aceserver_port{'1ace'}=100100;
    $main::aceserver_port{'20ace'}=100101;
    $main::aceserver_port{'22ace'}=100102;
    $main::aceserver_port{'6ace'}=100103;
    $main::aceserver_port{'Xace'}=100104;
    $main::aceserver_host{'humace'}=$main::humaceserver_host;
    $main::aceserver_host{'1ace'}='humsrv1';
    $main::aceserver_host{'20ace'}='humsrv1';
    $main::aceserver_host{'22ace'}='humsrv1';
    $main::aceserver_host{'6ace'}='humsrv1';
    $main::aceserver_host{'Xace'}='humsrv1';
    return 0;
}

# read information about sequences in pace into an hash of hashes
sub get_pace_sequence {
    my($type,$rhclone,$clone)=@_;

    my($pace_sequence_cache,$nc,$out);

# hard definitions
# location of pace sequence cache file
    $pace_sequence_cache='/nfs/disk100/humpub/th/pace.sequence.cache';
    $nc=0;
    $out='';

# either read from cache or read from pace and rewrite cache
    if($type eq 'cache'){
	open(TMP,$pace_sequence_cache) || die "cannot open $pace_sequence_cache";
	while(<TMP>){
#               1     2          3          4      5          6          7          8     9     10
	    if(/(.+)\t([\w\-]+)\t([\w\-]*)\t(\w*)\t([\w\-]*)\t([\w\-]*)\t([\w\-]*)\t(.*)\t(.*)\t(.*)/){
		next if($clone && $clone ne $1);
		$nc++;
		$$rhclone{$1}->{'chr'}=$2;
		$$rhclone{$1}->{'dfin'}=$3;
		$$rhclone{$1}->{'nana'}=$4;
		$$rhclone{$1}->{'dana'}=$5;
		$$rhclone{$1}->{'disub'}=$6;
		$$rhclone{$1}->{'dlsub'}=$7;
		$$rhclone{$1}->{'sb'}=$8;
		$$rhclone{$1}->{'sp'}=$9;
		$$rhclone{$1}->{'contig'}=$10;
	    }else{
		$out.=$_;
	    }
	}
	close(TMP);
	if($out){
	    print "Following lines not read from cache file:\n----\n$out----\n";
	}
    }else{
	die "Type $type not recognised in get_pace\n";
    }
    return $nc;

}

sub clone_annotator{
    my($clone)=@_;
    my(%clone);
    &get_pace_sequence('cache',\%clone,$clone);
    return $clone{$clone}->{'nana'};
}

# return the paceclonename for a clone name (effectively removing
# the last character of the name if required)
sub get_pace_name{
    my($clone,$rhclone)=@_;
    my($clonep);
    $clonep=$clone;
    if(!$$rhclone{$clonep}){
	chop($clonep);
	if(!$$rhclone{$clonep}){
	    $clonep='';
	}
    }
    return $clonep;
}

# read information about sequences in humace into an hash of hashes
sub get_humace_gseq{
    my($type,$gseq,$clone)=@_;
    my($humace_gseq_cache,$nc);
    
# hard definitions
# location of pace sequence cache file
    $humace_gseq_cache='/nfs/disk100/humpub/th/humace.gseq.cache';
# ports
    my($host,$port)=($main::humaceserver_host,$main::humaceserver_port);
    &get_gseq($type,$gseq,$humace_gseq_cache,$clone,$host,$port);
}

sub get_gseq{
    my($type,$gseq,$humace_gseq_cache,$clone,$host,$port)=@_;
    my($nc);
    $nc=0;

# read from db and update cache
    if($type eq 'read'){
	
# read from db
	unless($host){
	    die "host not defined";
	}
	my($db,$clone2,$chr,$cobj,$mess);
	$db=&slowConnect($host,$port,\$mess,4) || die "$mess";
	my @clones=$db->list('Genome_sequence','*');
	$nc=0;
	foreach $cobj (sort @clones){
	    $clone2="$cobj";
	    $nc++;
	    print "[$nc] $clone2\n";
	    if(($chr)=$cobj->Chromosome){
		$$gseq{$clone2}->{'chr'}=$chr;
	    }else{
		$$gseq{$clone2}->{'chr'}='-';
	    }
	}
	
# create new cache file
	open(TMP,">$humace_gseq_cache.new") || die "cannot open $humace_gseq_cache.new";
	foreach $clone2 (sort keys %$gseq){
	    $chr=$$gseq{$clone2}->{'chr'};
	    print TMP "$clone2\t$chr\n";
	}
	close(TMP);
	rename($humace_gseq_cache,"$humace_gseq_cache.old");
	rename("$humace_gseq_cache.new",$humace_gseq_cache);
	foreach $clone2 (sort %$gseq){
	    if($clone && $clone ne $clone2){
		delete $$gseq{$clone2};
	    }
	}

# read from cache or read from pace and rewrite cache
    }elsif($type eq 'cache'){
	open(TMP,$humace_gseq_cache) || die "cannot open $humace_gseq_cache";
	while(<TMP>){
	    if(/(.+)\t(\w+)/){
		next if($clone && $clone ne $1);
		$nc++;
		$$gseq{$1}->{'chr'}=$2;
	    }
	}
	close(TMP);
    }else{
	die "Type $type not recognised in get_humace_gseq\n";
    }
    return $nc;

}

# return a standard html header with links back to main sanger page
# showing title
sub hahtmlheader{
    my($title)=@_;
    my($out);
    $out=<<ENDOFTEXT;
<html>
<head><title>$title</title></head>
<body text="#000000" BGCOLOR="#FFFFFF">
<P><A HREF="http://www.sanger.ac.uk/"><IMG SRC="/LocalUsers/th/sanger-centre.gif" 
BORDER=0 HEIGHT=29 WIDTH=236 ALIGN=RIGHT></A>
<IMG SRC="/LocalUsers/th/helix-r.GIF" HEIGHT=23 WIDTH=55 ALIGN=LEFT>
<BR clear=all>
<HR></P>
<h2>$title</h2>
ENDOFTEXT
    return $out;
}

# return a standard html footer with links back to main sanger page
# and mail links to a use + date
sub hahtmlfooter{
    my($id,$date,$np)=@_;
    my($out,$add,$i);
    $add="$id\@sanger.ac.uk";
    $out=<<ENDOFTEXT;
<address>$id, $date, <a href="mailto:$add">$add</a></address>
ENDOFTEXT
    if($np){
	$out.="<pre>";
	for($i=0;$i<$np;$i++){
	    $out.="\n";
	}
	$out.="<\/pre>";
    }
    $out.=<<ENDOFTEXT;
</body>
</html>
ENDOFTEXT
    return $out;
}

# return a standard html header with links back to main sanger page
# showing title
sub Schtmlheader{
    my($title,$id,$full)=@_;
    my($out);
    $out=<<ENDOFTEXT;
<!--#config timefmt="%d-%b-%C%y, %I:%M %p" -->
<!--#set var="title" value="$title" --> 
<!--#set var="icon" value="proj.gif" --> 
<!--#set var="author" value="$id" -->
<!--#exec cgi="/cgi-bin/header" -->
ENDOFTEXT
    if($full){
	$out.="<\/td><\/tr><\/table><\/center>";
    }
    return $out;
}

# return a standard html footer with links back to main sanger page
# and mail links to a use + date
sub Schtmlfooter{
    my($full)=@_;
    my($out);
    $out=<<ENDOFTEXT;
<!--#exec cgi="/cgi-bin/footer" -->
ENDOFTEXT
    if($full){
	$out.="<table><tr><td>";
    }
    return $out;
}

sub numerically{
    $main::a <=> $main::b;
}

# max:
sub max{
    my($i,$j)=@_;
    if($i>$j){
	return $i;
    }else{
	return $j;
    }
}

# min:
sub min{
    my($i,$j)=@_;
    if($i<$j){
	return $i;
    }else{
	return $j;
    }
}

sub num_comma{
    my($n)=@_;
    my($nn);
    return $n unless $n=~/\d+/;
    {
	if($n=~/^(.*)(\d{3})$/){
	    $n=$1;
	    if($nn){
		$nn=",$nn";
	    }
	    $nn="$2$nn";
	    redo;
	}
	if($n ne ''){
	    $nn="$n,$nn";
	}
	return $nn;
    }
}
	

# expects following parameters to be defined in $main::
#   $humaceserver_host
#   $humacegserver_port
# returns a number of lists, indexed by the hp.pl commands that are required to be run:
#   %run
#   %update
#   $mdiff - oldest analysis
#   $mess - messages
# not passed:
#   %unloaded (info only - loaded by script)
#   %compress (info only - compressed by script)
# bugs:
#   mdiff does not take into account versions without date info (embl, etc).
sub raw_analysis_state{
    my($config_file,$clonename,$load,$compress,
       $rhrun,$rhupdate,$rhincomplete,$rfl,$rmdiff,$rmess,$rhdocompress,
       $rexedir,$rhacetype,$rhdbdatediff,$rhcurrent)=@_;
    my(%switch,$mess,%command,%dbver,$com,$acefilename,%acefilename,$clone,$db,
       $anadir,$tmpdir,@files,$lfile,$file,$n,$newanadir,$col,@col,
       %acefiles,$acefiles,$date,$found,%found,%compress,%unloaded,%missing,$phead,$pbody,
       $t1,$t2,$t3,$dayf,$diff,$day,$dir,$fadd,$perr,%foundtag,$tmp,$mdate,$fresolve,
       $res,$clonefile,$err);
    %$rhrun=();
    %$rhupdate=();
    %$rhincomplete=();
    %$rhdocompress=();
    %$rhcurrent=();
    $$rmess='';

# build list of what files are expected: %acefilename
# get list of configs
    &Read_Config($config_file,\%switch,\$mess);

# files should have names without Em: (unix)
# [all internal calculations in this subroutine are carried out without Em:
# since Analysis_files are expected to be without Em as are files themselves
# however script can cope with both types of error]
    $clonefile=&clonefile($clonename);

# construct name of file from config list [without Em:]
    &GetAceFileName($clonefile,\%switch,\%command,\%dbver);

# build a reverse list (files -> commands)
#    print "\nExpect\n" if $opt_d;
    foreach $com (keys %command){
#	print "$com -> $command{$com}\n" if $opt_d;
	$acefilename{$command{$com}}=$com;
    }
    $$rmess.="  ".scalar(keys %acefilename)." acefiles expected\n";

# get list of Analysis_files from humace: 
    $anadir='';
    $err=&GetAnalysisFiles($clonename,$rmess,\$anadir,\%found,$db,
			   $main::humaceserver_host,$main::humacegserver_port);
    if($err){
	return 2;
    }

# convert analysis directory stored in humace into full path and check it
    if($main::humpub_home && $anadir=~/~humpub(\/.*)/){
	$anadir="$main::humpub_home$1";
    }else{
	$anadir=glob $anadir;
    }
    unless($anadir){
	$$rmess.="  ERROR: Analysis_directory for clone $clonename not found in humace\n";
	return 1;
    }

# get dates as readin to humace
# (for working out range of dates for each search type)
    foreach $file (keys %found){
	$date=$found{$file};
	if($file=~/^$clonename\.(.*)\.ace$/){
	    $$rhacetype{$1}->[0]++;
	    if($mdate=$$rhacetype{$1}->[1]){
		if(&datediff($mdate,$date)<0){
		    $$rhacetype{$1}->[1]=$date;
		}
	    }else{
		$$rhacetype{$1}->[1]=$date;
	    }
	    if($mdate=$$rhacetype{$1}->[2]){
		if(&datediff($mdate,$date)>0){
		    $$rhacetype{$1}->[2]=$date;
		}
	    }else{
		$$rhacetype{$1}->[2]=$date;
	    }
	}
    }

# check if anadir exists
# (should also write acefile to update if directory is wrong)
    unless(-d $anadir){
	$$rmess.="  WARN: Analysis directory in humace ($anadir) does not exist\n";
    }
    if($anadir=~/(.*)\/\d+$/){
	$tmpdir=$1;
	opendir(DIR,"$tmpdir") || die "cannot open $tmpdir";
	@files=readdir(DIR);
	closedir(DIR);
	$lfile='';
	foreach $file (@files){
	    if($file=~/^\d+$/){
		$n++;
		$newanadir="$tmpdir/$file";
		$lfile.="  $file\n";
	    }
	}
	if($n==1){
	    unless($anadir eq $newanadir){
		$anadir=$newanadir;
		$$rmess.="  WARN: Analysis directory was resolved to be $anadir\n";
		$$rmess.="  WARN: Please update entry in humace\n";
	    }
	}else{
	    $$rmess.="  ERROR: $n directories in $tmpdir - could not resolve\n";
	    $$rmess.=$lfile;
	    return 1;
	}
    }else{
	$$rmess.="  ERROR: analysis directory checks could not be carried out\n";
	return 1;
    }
    $$rexedir=$anadir;

# get list of files in rawdata directory: %acefiles
    $n=0;
    if(-d "$anadir/rawdata"){
	opendir(DIR,"$anadir/rawdata") || die "cannot open $anadir";
	@files=readdir(DIR);
	closedir(DIR);
	foreach $file (@files){
# if file has Em: prefix, ignore
# [this should be historical, before it was enforced that all files for clone Em:xx were
#  named xx]
	    if($file=~/^Em:(.*\.ace)$/){
		$acefiles{$1}=1;
	    }elsif($file=~/\.ace$/){
		$acefiles{$file}=1;
	    }
	    $n++;
# get dates from raw files only (gz will have different date)
	    if($file=~/$clonefile\.(.*)\.ace$/){
		$$rhacetype{$1}->[0]++;
		$date=&fileDateAcedb("$anadir/rawdata/$file");
		if($mdate=$$rhacetype{$1}->[1]){
		    if(&datediff($mdate,$date)<0){
			$$rhacetype{$1}->[1]=$date;
		    }
		}else{
		    $$rhacetype{$1}->[1]=$date;
		}
		if($mdate=$$rhacetype{$1}->[2]){
		    if(&datediff($mdate,$date)>0){
			$$rhacetype{$1}->[2]=$date;
		    }
		}else{
		    $$rhacetype{$1}->[2]=$date;
		}
	    }
	}
    }
    $$rmess.="Found $n ace files in:\n  $anadir\n";

# three lists:
# calculated files (union of %found and %acefiles)
# unloaded files (%acefiles not in %found)
# loaded files not yet compressed (%acefiles in %found)

    foreach $acefiles (keys %acefiles){
	if($found{$acefiles}){
	    $compress{$acefiles}=1;
	}else{
	    $unloaded{$acefiles}=1;
	    $found{$acefiles}=1;
	}
    }

# first scan for exact matches...
    $phead="\nAnalysis results that are loaded and uptodate\n";
    $pbody='';
    foreach $acefilename (keys %acefilename){
	if($found{$acefilename}){
	    $foundtag{$acefilename}=1;
	    $$rhcurrent{$acefilename}=1;
	    $pbody.="$acefilename\n";
	}else{
	    $missing{$acefilename}=1;
	}
    }
    if($pbody){
	$$rmess.=$phead.$pbody;
    }


# try other matches
    $phead="\nDatabases searched are not current - reanalysis required\n";
    $pbody='';
    $$rmdiff=0;
  LOOP:
    foreach $acefilename (keys %missing){
# date updated files (dbEST, STS, GSS)
	if($acefilename=~/^(\w+\.)(db(EST|STS|GSS))(\d+)(\.\w*\.*ace)$/){
	    $t1=$1;
	    $t2=$2;
	    $day=&usa_date($4);
	    $t3=$5;
	    foreach $found (keys %found){
		if($found=~/^$t1$t2(\d+)$t3$/ || $found=~/^$t2(\d+)$t3$/){
		    $dayf=&usa_date($1);
		    $diff=$day-$dayf;
# list update requirements (commands)
		    if($diff!=0){
			$pbody.="$found -> $acefilename ($diff days old)\n";
			$$rhupdate{$acefilename{$acefilename}}="$acefilename:$found";
			$$rhcurrent{$found}=1;
		    }
# record oldest
		    if($diff>$$rmdiff){$$rmdiff=$diff;}
# record resolved
		    delete $missing{$acefilename};
		    $foundtag{$found}=1;
		    next LOOP;
		}
	    }
# version updated files
# acefilename is what it should be; found is what is present
	}elsif($acefilename=~/$clonefile\.(\w+?)([\d\.]+)\.ace$/){
	    $t1=$1;
	    $t2=$2;
	    foreach $found (keys %found){
		if($found=~/$clonefile\.$t1([\d\.]+)\.ace$/){
		    if($found ne $acefilename){
			if($diff=$$rhdbdatediff{$t1}->{$1}){
			    if($diff>$$rmdiff){$$rmdiff=$diff;}
			}else{
			    $diff='???';
			}
			$pbody.="$found -> $acefilename ($diff days old)\n";
			$$rhupdate{$acefilename{$acefilename}}="$acefilename:$found";
			$$rhcurrent{$found}=1;
		    }
		    delete $missing{$acefilename};
		    $foundtag{$found}=1;
		    next LOOP;
		}
	    }
	}
    }
    if($pbody){
	$$rmess.=$phead.$pbody;
    }

# check remaining missing against list of files in directory
    foreach $acefilename (keys %missing){
	$$rhrun{$acefilename{$acefilename}}=$acefilename;
    }

# any ace file for loading, must have foundtag to be valid
    foreach $acefilename (keys %unloaded){
	unless($foundtag{$acefilename}){
	    delete $unloaded{$acefilename};
	}
    }
    
    $phead="\nNot loaded";
    $$rfl=0;
    unless($load){
	$phead.=" - loading required\n";
	$perr='';
    }else{
	$phead.=" - files are being copied for reloading\n";
	$perr=' [not loaded]';
    }
    $pbody='';
    foreach $acefilename (keys %unloaded){
	$pbody.="$acefilename\n";
	$$rhcurrent{$acefilename}=1;
	$dir="$anadir/rawdata";
	$file="$dir/$acefilename";
# a zero length file is either a fatal error or a file in progress
	if(-z $file){
	    $pbody.="ERROR$perr: $acefilename is empty - in progress?\n";
	    $$rhincomplete{$acefilename{$acefilename}}=$acefilename;
	}else{
	    $fadd=0;
	    open(ACE,"tail $file |") || die "cannot tail $file $!\n";
	    while(<ACE>){
		if(/^Analysis_files/){
		    $fadd=1;
		    last;
		}
	    }
	    close(ACE);
	    if($fadd){
		if($load){
		    &file2humace_queue($acefilename,$dir,$main::humace_queue);
		}
	    }else{
# a new file without an analysis tag is either in progress or a fatal error
		$$rhincomplete{$acefilename{$acefilename}}=$acefilename;
# in old cases prior to transaction control
# may be loaded - try to resolve this way
		$fresolve=0;
		if($fresolve){
		    $res=&resolve_incomplete($dir,$acefilename,$acefilename{$acefilename},
					     $load,$clonename,$db,\$mess);
		    $pbody.=$mess;
		}else{
		    $pbody.="ERROR$perr: $acefilename has no Analysis_files tag - in progress?\n";
		}
	    }
	}
    }
    if($pbody){
	$$rmess.=$phead.$pbody;
	$$rfl=1;
    }

    $phead="\nMissing - another analysis run is required\n";
    $pbody='';
    foreach $acefilename (keys %missing){
	$pbody.="$acefilename\n";
    }
    if($pbody){
	$$rmess.=$phead.$pbody;
    }
    
    $phead="\nLoaded";
    unless($compress){
	$phead.=" - file in the analysis directory can be removed/compressed\n";
    }else{
	$phead.=" - files are being compressed\n";
    }
    $pbody='';
    foreach $acefilename (keys %compress){
	$pbody.="$acefilename\n";
	if($compress){
	    $file="$anadir/rawdata/$acefilename";
	    if(-z $file){
		$pbody.="ERROR: $acefilename is empty - not compressed - in progress?\n";
	    }else{
		$$rhdocompress{$file}=1;
	    }
	}
    }
    if($pbody){
	$$rmess.=$phead.$pbody;
    }
    return 0;
}

# for a clone, returns a hash (\%found) of names of files readinto humace, 
# with dates in ace format
# also returns analysis directory as stored in humace
sub GetAnalysisFiles{
    my($clonename,$rmess,$ranadir,$rhfound,$db,
       $host,$port)=@_;
    %$rhfound=();
    my($clone,@col,$col,$date,$file,$n);

# try to use an existing connection, unless not passed port/host in which 
# case assumed that $db is valid
    if($port && $host){
	&reconnect(\$db,$host,$port,$rmess,4) && return 2;
    }
    $db->date_style('ace');

# find analysis directory, so we can check what files are there: %acefiles
    ($clone)=$db->list('sequence',$clonename);
#    print "|$clone|\n";
    unless($clone){
	$$rmess.="  ERROR: Clone $clonename not found in humace\n";
	return 1;
    }

# get analysis directory
    ($$ranadir)=$clone->Analysis_directory;

# get list of files read in: %found
    if($clone->Analysis_files){
#	print "\nFound\n" if $opt_d;
	@col=$clone->Analysis_files;
	$n=0;
	foreach $col (@col){
	    $file=$col->name;
	    $file=&clonefile($file);
	    ($date)=$col->row(1);
#	    print "$file [$date]\n";
	    $n++ unless $$rhfound{$file};

# fix date for missing leading zeros (do we care?)
	    if($date=~/^(\d+)\-(\d)\-(\d+)$/){
		$date="$1-0$2-$3";
	    }
	    if($date=~/^(\d+\-\d+)\-(\d)$/){
		$date="$1-0$2";
	    }

	    $$rhfound{"$file"}=$date;
#	    $$rhfound{"$file"}=1;

	}
	$$rmess.="  $n ace files already readin to humace\n";
    }else{
	$$rmess.="  no ace files already readin to humace\n";
    }
    return 0;
}

# gets date of a file and converts to acedb format
sub fileDateAcedb{
    my($file)=@_;
    my(@array,$yr,$mon,$day,$date);
    if(!-e $file){
	print "ERROR: $file not found in humpub::fileDateAcedb\n";
    }
    @array=stat($file);
    @array=localtime($array[9]);
    $yr=$array[5];
# 2000 year fix
    unless($yr>99){
	if($yr>70){
	    $yr="19$yr";
	}else{
	    $yr="20$yr";
	}
    }
    $mon=$array[4]+1;
    unless($mon>9){$mon="0$mon";}
    $day=$array[3];
    unless($day>9){$day="0$day";}
    $date="$yr-$mon-$day";
    return $date;
}

# resolve incomplete list:
# requires to 1) check if a object/tag combination in this file is in the acedb database
# and 2) create a dummy file containing Analysis_files and load this
sub resolve_incomplete{
    my($rawdir,$acefile,$com,$load,$clonename,$db,$rmess)=@_;
    my(%check_read,$tagfile,$tagdir,$file,$clone,$value,$date,
       $ffound,$code,$tags,@tags,$objname,$value,%value,@value,$ntags,$tmp);
    $file="$rawdir/$acefile";
    $tagfile="tag.$acefile";
    $tagdir="/tmp";

# definitions
    %check_read=(
		 do_genscan=>{
		     objname=>'Sequence +($clonename\.(RC|)\.GENSCAN\.\d+)',
		     tag=>'Source_exons',
		 },
		 do_solovyev=>{
		     objname=>'Sequence +($clonename\.FGENEH\.)',
		     tag=>'(Source_exons) +(\d+)',
		 },
		 );

    $$rmess='';

# check if we know how to check this command type
    unless($check_read{$com}){
	$$rmess.="no check_read for [$com]\n";
	return 1;
    }

# look for a match in the ace file
    open(TMP,$file) || die "cannot open $file";
    $ffound=0;
    $code=<<ENDOFCODE;
while(<TMP>){
    if(/^$check_read{$com}->{'objname'}/){
	\$ffound=1;
	\$objname=\$1;
    }elsif(/^ *\$/ && \$ffound==1){
	\$ffound=0;
	last;
    }elsif(\$ffound==1 && /$check_read{$com}->{'tag'}/){
	\$tmp=\$1;
	push(\@tags,\$2);
    }
}
ENDOFCODE
    eval $code;
    die "FATAL eval: $@" if $@;
    close(TMP);

# try to check
# first scan file for cases which match
# now look for those cases in acedb
    ($clone)=$db->list('sequence',$objname);
    print "|$tmp|\n";
    @value=$clone->search($tmp);
    undef %value;
    foreach $value (@value){
	$value{$value}=1;
    }
    $ffound=0;
    $ntags=0;
    foreach $tags (@tags){
	$ntags++;
	unless($value{$tags}){
	    $ffound++;
	}
    }
    if($ffound==0){
	if($load){
	    $$rmess.="INFO: had been loaded into acedb - loading transaction file\n";
	    $date=&fileDateAcedb($file);
	    &append_af($clonename,"$tagdir/$tagfile",$date,1,0,$acefile);
	    &file2humace_queue($tagfile,$tagdir,$main::humace_queue);
	    unlink($tagfile);
	}else{
	    $$rmess.="INFO: had been loaded into acedb - transaction tag needs to be loaded\n";
	}
    }elsif($ntags==$ffound){
	if($load){
	    $$rmess.="WARN: not loaded into acedb - loading\n";
	    &append_af($clonename,"$rawdir/$acefile",$date,0,1);
	    &file2humace_queue($acefile,$rawdir,$main::humace_queue);
	}else{
	    $$rmess.="WARN: not loaded into acedb - loading required\n";
	}
    }else{
	$$rmess.="ERROR: $file partially loaded into acedb: $ffound out of $ntags not loaded\n";
    }

# if check is ok, create and load tag file
    return 0;
}

# get acefilename corresponding to functions in haceprep.cfg file
sub GetAceFileName{
    my($clone,$rhswitch,$rhacefilename,$rhdbver)=@_;
    my($switch,$dbver,%fixed);
    %$rhacefilename=(
		     );

# get version names
    &getdbver($rhdbver);

# build fixed names
    no strict 'vars';
    %fixed=(
	    'solovyev'=>'solovyev',
	    'grail'=>'grail13',
	    'genscan'=>'genscan',
	    'xpound'=>'xp',
	    'est_clusters'=>$$rhdbver{'dbest'}.".clusters",
	    'seq2ace'=>'',
	    'RepeatMasker'=>'RepeatMasker',
	    'mask'=>'masking',
	    'cpg'=>'cgi',
	    'gc'=>'gc-profile',
	    'halfwise'=>'HalfWise_2.1.12b'
	    );

# loop over known definition to see if config set
    foreach $dbver (keys %$rhdbver){
	$switch="do_$dbver";
	if($$rhswitch{$switch}){
	    $$rhacefilename{$switch}="$clone.".$$rhdbver{$dbver}.".ace";
	}
    }
# loop over fixed definitions
    foreach $dbver (keys %fixed){
	$switch="do_$dbver";
	if($$rhswitch{$switch}){
	    if($fixed{$dbver}){
		$$rhacefilename{$switch}="$clone.".$fixed{$dbver}.".ace";
	    }else{
		$$rhacefilename{$switch}="$clone.ace";
	    }
	}
    }
    return 0;
}

# get acetag associated with functions in haceprep.cfg file
sub GetAceTag{
    my($rhswitch,$rhacetag)=@_;
    my($switch,$com,%fixed,$err);
    %$rhacetag=(
		);

# build fixed names
    no strict 'vars';
    %fixed=(
	    'swir'=>'Pep_homol',
	    'dbest'=>'EST_homol',
	    'dbsts'=>'STS_homol',
	    'dbgss'=>'GSS_homol',
	    'vertrna'=>'vertebrate_mRNA_homol',
	    'embl'=>'DNA_homol',
	    );

# loop over fixed definitions
    $err=0;
    foreach $switch (keys %$rhswitch){
	if($switch=~/do_(\w+)/){
	    $com=$1;
	    if($fixed{$com}){
		$$rhacetag{$switch}=$fixed{$com};
	    }else{
		$$rhacetag{$switch}='';
		$err=1;
	    }
	}
    }
    return $err;
}

# read haceprep config file
# -> $config (file or command string)
# <- \%switch (set to 1 or 0)
# <- \$mess   (printable string of settings there were read)
sub Read_Config{
    my($config,$rhswitch,$rmess)=@_;
    my($name, $val, $rest, $value, $lines);
    my @lines;
# Open file handle for reading config file
    if(!-e $config && $config=~/^do_/){
	$$rmess="Parsing Config parameters\n";
	foreach $lines (split(/[;,]/,$config)){
	    ($name,$val)=(split(/=/,$lines));
	    unless($val){
		$val=1;
	    }
	    $$rhswitch{$name}=$val;
	    $$rmess.="$name=$val\n";
	}
    }else{
	$$rmess="Reading Config file\n";
	open (CONFIG, "$config") or die "Couldn't open $config : $!\n";
	@lines = <CONFIG>;
	close (CONFIG);
	foreach $lines (@lines) {
# ignore comments
	    next if /^#/;
	    ($val, $rest) = split('#', $lines, 2);
	    ($name, $value) = split ('=', $val, 2);
# Remove whitespace
	    $name =~ s/\s//g;
	    $value =~ s/\s//g;
	    $$rhswitch{$name} = $value;	
	    if ($value eq "1") {
		$$rmess.="$name\n";
	    }
	}
    }
    return 0;
}

sub Expand_Config{
    my($config,$rhswitch,$rarules,$rmess)=@_;
    my(%switch2,$mess,%rules,$rule,$name,$expand);
# unpack rules
    foreach $rule (@$rarules){
	push(@{$rules{$$rule{'name'}}},@{$$rule{'require'}},@{$$rule{'option'}});
    }
# read default config
    &Read_Config($config,\%switch2,\$mess);
# step through existing and expand
    foreach $name (keys %$rhswitch){
	if($rules{$name}){
	    foreach $expand (@{$rules{$name}}){
# check if already set
		unless($$rhswitch{$expand}){
		    if($switch2{$expand}){
			$$rhswitch{$expand}=$switch2{$expand};
			$$rmess.="$name->$expand=$switch2{$expand}\n";
		    }
		}
	    }
	}
    }
}

# subroutine specifies list of name (do_NAME) file relationships
# enter elements into hash
sub getdbver{
    my($rname)=@_;
    my(%dna,%prot,$name);

# define fixed names
    $$rname{'cgidb'} = 'cgidb5';
    $$rname{'finished'} = 'finished';
    $$rname{'public_unfin'} = 'public_unfinished';
    $$rname{'private_unfin'} = 'private_unfinished';
    $$rname{'prereleased'} = 'pre-released';
    $$rname{'sanger_sts'} = 'sanger_STS';

# maps filename (in versions file) to variable name
    %dna=(
	  'dbEST'=>'dbest',
	  'dbSTS'=>'dbsts',
	  'dbGSS'=>'dbgss',
	  'embl_vertrna'=>'vertrna',
	  'embl_other'=>'embl',
	  'emnew_other'=>'emnew',
	  'emnew_vertrna'=>'vertrnanew',
	  );

    %prot=(
	   'swir'=>'swir',
	   );

# setup blast versions: dna
    open (VER, "/nfs/disk100/pubseq/blastdb/blastversions.dna") ||
	die "Can't open blastversions.dna : $!\n";
    while (<VER>) {
	foreach $name (keys %dna){
	    if (/\s+$name\s+(\S+)/) {
		$$rname{$dna{$name}} = "$name$1";
	    }
	}
    }
    close VER;

# setup blast versions: protein
    open (VER, "/nfs/disk100/pubseq/blastdb/blastversions.prot") ||
	die "Can't open blastversions.dna : $!\n";
    while (<VER>) {
	foreach $name (keys %prot){
	    if (/\s+$name\s+(\S+)/) {
		$$rname{$prot{$name}} = "$name$1";
	    }
	}
    }
    close VER;
    return 0;
}

# check status of a job
sub check_bstat{
    my($bjob)=@_;
    my($stat);
    open(TMPTMP,"bjobs $bjob |") || die "could not run bjobs $!\n";
    while(<TMPTMP>){
	if(/^$bjob\s+\w+\s+(\w+)/){
	    $stat=$1;
	}
    }
    close(TMPTMP);
    if($stat eq 'EXIT'){
	return 1;
    }else{
	return 0;
    }
}

# utility
# if $logfile exists, rename it to $logfile.n, incrementing n until
# a non existant file is found
# returns a message if a rename occured
sub save_old_file{
    my($logfile,$rmess,$dir)=@_;
    my($n,$logfilenew,$logfile2);
    $$rmess='';
    if($dir){
	if($dir=~/\/$/){
	    chop($dir);
	}
	unless($logfile=~/^(.*)\/([^\/]+)$/){
	    $$rmess="logfile could not be parsed into dir/file [$logfile]";
	    return 1;
	}
	$logfile2="$1/$dir/$2";
    }else{
	$logfile2=$logfile;
    }
    if(-e $logfile){
	$n=1;
	{
	    $logfilenew="$logfile2.$n";
	    if(-e $logfilenew){
		$n++;
		redo;
	    }
	    rename($logfile,$logfilenew);
	    $$rmess="WARN: old logfile found and renamed: $logfile->$logfilenew\n";
	}
    }
}

# utility
# if $logfile exists and is too long (lines<$maxlines)
# save first all except last minlines lines to an old directory, with renaming
sub save_append_file{
    my($logfile,$rmess,$maxlines,$minlines)=@_;
    my($n,$ns,$dir,$file,$nlogfile);
# set defaults
    $maxlines=5000 unless($maxlines);
    $minlines=500 unless($minlines);
    $$rmess='';
# check file
    if(!-e $logfile){
	$$rmess="logfile not found [$logfile]";
	return 1;
    }
    open(TIN,$logfile) || die "cannot open $logfile";
    $n=0;
    while(<TIN>){
	$n++;
    }
    close(TIN);
# return if short enough
    return 0 if $n<$maxlines;
    $ns=$n-$minlines;
# check for directory
    unless($logfile=~/^(.*)\/([^\/]+)$/){
	$$rmess="logfile could not be parsed into dir/file [$logfile]";
	return 1;
    }
    $dir=$1;
    $file=$2;
    $dir.='/old';
# make old directory if it doesn't exist
    if(!-d $dir){
	if(-e $dir){
	    $$rmess="$dir exists but is not a directory!!";
	    return 1;
	}
	mkdir ($dir,0777);
    }
    $nlogfile="$dir/$file";
    &save_old_file($nlogfile);
    rename($logfile,"$logfile.tmp");
    open(TOUT,">$nlogfile") || die "cannot open $nlogfile";
    open(TIN,"$logfile.tmp") || die "cannot open $logfile";
    $n=0;
    while(<TIN>){
	print TOUT;
	$n++;
	if($n==$ns){
	    close(TOUT);
	    open(TOUT,">$logfile") || die "cannot open $logfile";
	}
    }
    close(TIN);
    close(TOUT);
    unlink("$logfile.tmp");
    $$rmess="logfile shortened ok";
    return 0;
}

# utility
# if $logfile exists remove duplicate null records
sub compress_append_file{
    my($logfile,$rmess)=@_;
    my($out1,$out2,$f,$t1,$line,$n);
    $$rmess='';
# check file
    if(!-e $logfile){
	$$rmess="logfile not found [$logfile]";
	return 1;
    }
# parse log file into paragraphs
    open(TIN,$logfile) || die "cannot open $logfile";
    $out1='';
    $out2='';
    $f=0;
    while(<TIN>){
	if(/^\s*$/){
	    $f=1;
	}elsif($f==1){
	    $out1.=$out2;
	    $out2='';
	    $f=0;
	}
	$out2.=$_;
    }
    close(TIN);
# check last para - exit if no tag indicating a null log
    unless($out2=~/ok\s+\[0\]/s){
	return 0;
    }
    rename($logfile,"$logfile.tmp");
    open(TOUT,">$logfile") || die "cannot open $logfile";
    print TOUT $out1;
    $f=0;
    $n=0;
    foreach $line (split(/\n/,$out2)){
	$n++;
	if($n==1){
	    print TOUT "$line\n";
	    next;
	}elsif($line=~/^\[(\d+)\]$/){
	    $t1=$1;
	    $t1++;
	    print TOUT "[$t1]\n";
	    $f=1;
	}
	last;
    }
    unless($f){
	print TOUT "[1]\n";
    }
    close(TOUT);
    unlink("$logfile.tmp");
    return 0;
}

# get current date in yyyy-mm-dd format,
# which is acedb format.
sub getdate{
    my($ftime)=@_;
    my($time);
    # JRGR -- changed because was returning
    # tomorrow's date, since mday has range
    # 1..31
    my ($sec,$min,$hr,$mday,$mon,$year) = (localtime)[0,1,2,3,4,5];
    if($ftime){
	if(length($hr)==1){$hr="0$hr";}
	if(length($min)==1){$min="0$min";}
	if(length($sec)==1){$sec="0$sec";}
	$time=" $hr:$min:$sec";
    }
    $year = 1900 + $year;
    $mon = ('01'..'12')[$mon];   #  mon starts at 0
    $mday = ('00'..'31')[$mday]; # mday starts at 1!
    return "$year-$mon-$mday$time";
}

# converts YYYY-MM to YYYY, Month (assumes MM = 01-12)
sub monthyear{
    my($date)=@_;
    my($year,$tm,$mon);
    if($date=~/^(\d+)\-(\d+)/){
	$year=$1;
	$tm=$2-1;
    }else{
	die "$date does not parse in monthyear";
    }
    $mon=('January','February','March','April','May','June',
	  'July','August','September','October','November','December')[$tm];
    return($year,$mon);
}

# convert YYMMDD or YYYYMMDD -> yyyy-mm-dd
sub file2date{
    my($efiled)=@_;
    my($efiledt);
    if(length($efiled)==8){
	if($efiled=~/(\d{4})(\d{2})(\d{2})/){
	    $efiledt="$1-$2-$3";
	}else{
	    die "$efiled can't be parsed $!\n";
	}
    }else{
	if($efiled=~/(\d{2})(\d{2})(\d{2})/){
	    if($1 > 70){
		$efiledt="19$1-$2-$3";
	    }else{
		$efiledt="20$1-$2-$3";
	    }
	}else{
	    die "$efiled can't be parsed $!\n";
	}
    }
    return $efiledt;
}

# convert DD-MON-YYYY or DD-MON-YYYY -> yyyy-mm-dd
sub convertDate{
    my($efiled)=@_;
    my($efiledt,%mon,$mon);
    %mon=('JAN','01','FEB','02','MAR','03',
	  'APR','04','MAY','05','JUN','06',
	  'JUL','07','AUG','08','SEP','09',
	  'OCT','10','NOV','11','DEC','12');
    if($efiled=~/(\d{2})\-(\w{3})\-(\d{2,4})/){
# convert months
	unless($mon=$mon{$2}){
	    die "$efiled cannott be parsed [$2] $!\n";
	}
# fix y2k
	if($3 > 999){
	    $efiledt="$3-$mon-$1";
	}elsif($3 > 70){
	    $efiledt="19$3-$mon-$1";
	}else{
	    $efiledt="20$3-$mon-$1";
	}
    }else{
	die "$efiled cannot be parsed $!\n";
    }
    return $efiledt;
}

# subtract yyyy-mm-dd from yyyy-mm-dd and return result in days
sub datediff{
    my($d1,$d2)=@_;
    my($days,$t1,$t2,$t3);
    if($d1=~/^(\d{4})\-(\d{1,2})\-(\d{1,2})/){
	$t1=$1;
	$t2=$2;
	$t3=$3;
	if($d2=~/^(\d{4})\-(\d{1,2})\-(\d{1,2})/){
	    $days=($1-$t1)*365+($2-$t2)*31+($3-$t3);
	    return $days;
	}else{
	    return 0;
	}
    }else{
	return 0;
    }
}

# converts usa date string MMDDYY (used in dbEST db names etc.)
# to number of days (approx)
sub usa_date{
    my($date)=@_;
    my($m,$d,$y,$day);
    ($m,$d,$y)=($date=~/(\d{2})(\d{2})(\d{2})/);
    $day=$y*365+$m*31+$d;
    return $day;
}

# appends Analysis_files lines to ace file
# if $new parameter set, creates a file with only these contents
# if $check parameter set, checks for existing lines and only appends
#                          if absent
# returns 0 if line was added
# returns 1 if no action
sub append_af{
    my($clone,$acefile,$acedate,$new,$check,$filename)=@_;
    my($file,$fadd);

# decide which is going to get written in Analysis_files tag
    if($filename){
	$file=$filename;
    }elsif($acefile=~/rawdata\/(.*)/){
	$file=$1;
    }else{
	$file=$acefile;
    }

    $fadd=1;
    if($check){
	if(&get_af($acefile)){
	    $fadd=0;
	}
    }
    if($fadd){
	if($new){
	    open(ACE,">$acefile") || die "cannot open $acefile $!\n";
	}else{
	    open(ACE,">>$acefile") || die "cannot open $acefile $!\n";
	}
	print ACE "\nSequence $clone\nAnalysis_files $file $acedate\n";
	close(ACE);
	return 0;
    }else{
	return 1;
    }
}

# find analysis_files tag and clone associated with
sub get_af{
    my($acefile)=@_;
    my($seq);
    open(ACE,"tail $acefile |") || die "cannot tail $acefile $!\n";
    while(<ACE>){
	if(/^Analysis_files/){
	    return $seq;
	}elsif(/^Sequence\s+\"*([^\"]+)/){
	    $seq=$1;
	    $seq=~s/\n//g;
	}
    }
    close(ACE);
    return;
}

# copy files to queue
sub file2humace_queue{
    my($file,$dir,$humace_queue)=@_;
    my($filet,$filef,$filen);
    $filen="$humace_queue/$file";
# don't copy if already in queue
    if(-e $filen){
	return 0;
    }
    $filet="$filen.COPYING";
    $filef="$dir/$file";
    copy($filef,$filet);
    rename($filet,$filen);
}

# sorting subroutine, so chr order is 1-n,X,Y,Unk
sub bychr{
    my($ta,$tb);
    $ta=$main::a;
    if($ta eq 'X'){
	$ta=23;
    }elsif($ta eq 'Y'){
	$ta=24;
    }elsif($ta eq 'Unk'){
	$ta=25;
    }
    $tb=$main::b;
    if($tb eq 'X'){
	$tb=23;
    }elsif($tb eq 'Y'){
	$tb=24;
    }elsif($tb eq 'Unk'){
	$tb=25;
    }
    ($ta <=> $tb);
}

# get list of jobs and runstate
# undef jobid add [0] = stat; [1] = queue
sub get_bjobs{
    my($rhjobid,$user,$queue)=@_;
    my($usercom,$queuecom);
    if($user){
	$usercom=" -u $user";
    }else{
	$usercom='';
    }
    if($queue){
	$queuecom=" -q $queue";
    }else{
	$queuecom='';
    }
    open(BJOBS,"bjobs $usercom $queuecom 2>&1 |");
    while(<BJOBS>){
#            jobid user  stat   queue  fromh  exech
	if(/^(\d+) +\w+ +(\w+) +(\w+) +(\w+) +(\w+|)/){
#                          0   1  2  3  4
	    $$rhjobid{$1}=[$2,$3,$5,$4,$5];
        }
    }
    close(BJOBS);
    return 0;
}

# get detailed info for jobs
# add [5] = com; [6] = cwd.
sub get_bjobs2{
    my($rhjobid)=@_;
    my($jobid,$string);
    foreach $jobid (keys %$rhjobid){
	open(BJOBS,"bjobs -l $jobid |");
	$string='';
	while(<BJOBS>){
	    chop;
	    if(/^\s+(\S.*)$/){
		$string.=$1;
	    }elsif(/^\S/){
		if($string=~/Command \<(.*)\>/){
		    ${$$rhjobid{$jobid}}[5]=$1;
	        }elsif($string=~/Execution CWD \<(.*)\>/){
		    ${$$rhjobid{$jobid}}[6]=$1;
		}
		$string=$_;
	    }
	}
	close(BJOBS);
    }
}

# restart an acedb server
sub restart_server{
    my($host,$port,$fpscheck)=@_;
    my($frun,$mess);
# check via ps if server is running, since if its not no point starting
# it to ask it to quit.  Of course, can only use this option on scripts that
# are running on the machine running the server
    if($fpscheck){
	$frun=0;
	open(TMPCOM,"ps -ef |") || die "cannot run ps";
	while(<TMPCOM>){
	    if(/rpc\.acedbd.*\s+$port\s+/){
		$frun=1;
	    }
	}
	close(TMPCOM);
    }else{
	$frun=1;
    }
    if($frun){
	my($command,$results,$db);
	unless($db=&slowConnect($host,$port,\$mess,4)){
	    print "$mess";
	    return 1;
	}
	$command="shutdown";
	$results=$db->raw_query($command);
# shuting down server, so kill connection object
	undef $db;
	return 0;
    }else{
	return -1;
    }
}

# convert Chr_x -> x
sub chrs{
    my($chr)=@_;
    my($chrs);
    if($chr=~/^Chr_(\w+)/){
	$chrs=$1;
    }else{
	$chrs='Unk';
    }
    return $chrs;
}

# get usage of a filesystem
sub get_df{
    my($dir,$rleft,$rpercent)=@_;
    my($ok)=1;
    open(DF,"df -k $dir 2>&1 |");
    while(<DF>){
# lib1:/humpub    90599408    81219656     7733552    92%    /nfs/disk100/humpub
	if(/^[\w\:\/]+ +\d+ +\d+ +(\d+) +(\d+)\% +[\w\/\-]+/){
	    $$rleft=$1;
	    $$rpercent=$2;
	    $ok=0;
        }
    }
    close(DF);
    return $ok;
}

# add an href to a string
sub add_href{
    my($tag,$name,$pre)=@_;
# rule based expansion of known abreviations
    if($pre eq 'efetch'){
	$pre="http://www.sanger.ac.uk/cgi-bin/seq-query?";
    }elsif($pre eq 'humace'){
	$pre="http://webace.sanger.ac.uk/cgi-bin/webace?display=text&db=humace&class=Sequence&object=";
    }elsif($pre=~/^acedb\d+$/){
	$pre="http://webace.sanger.ac.uk/cgi-bin/webace?display=text&db=$pre&class=Sequence&object=";
    }
# if name is unset defaults to tag
    unless(defined $name){$name=$tag;}
# if no pre and tag does not end in html, add it
    if($pre eq '' && ($tag!~/html$/ && $tag!~/html\#/)){
	$tag.=".html";
    }
    my($html);
    $html="<a href=\"$pre$tag\">$name<\/a>";
    return $html;
}

sub html_style{
    my($word,$style)=@_;
    return "<$style>".$word."<\/$style>";
}

# multiple attempts to connect to acedb
sub slowConnect{
    my($host,
       $port,
       $rmess,
       $maxtries) = @_;
# More than 20 min of trying
    $maxtries=40 unless $maxtries;
    my $db;
  ACECONNECT: {
      my $tries;
      until ($db) {
          if($db=Ace->connect(-host => $host,
			      -port => $port)){
#	      print "Created new Ace connection ok\n";
	      last ACECONNECT;
	  }
          if ($tries >= $maxtries){ 
              $$rmess="ERROR: Couldn't connect to $host on $port".
		  " after $tries retries:".Ace->error()."\n";
	      last ACECONNECT;
          }else{
#	      print "Ace connection attempted failed\n";
	  }
          $tries++;
          sleep 30;
      }
  }
#    print '|'.ref($db)."|\n";
    return $db;
}

# checks to see if connected, else tries to connect
sub reconnect{
    my($rdb,$host,$port,$rmess,$maxtries)=@_;
    unless($$rdb && ref($$rdb) eq 'Ace' && $$rdb->ping()){
	if($$rdb=&slowConnect($host,$port,$rmess,$maxtries)){
	    return 0;
	}else{
	    return 1;
	}
    }else{
#	print "Found existing Ace connection\n";
	return 0;
    }
}

# get an Em free version of $clone
sub clonefile{
    my($clone)=@_;
    my($clonefile);
    $clonefile=$clone;
    $clonefile=~s/^Em://;
    return $clonefile;
}

1;  # says use was ok
__END__

=head1 NAME

humpub

=head1 DESCRIPTION

Description for B<humpub>

A set of subroutine calls for use by scripts in humpub.  Does not use
an object model.

=head1 AUTHOR

B<Tim Hubbard> Email th@sanger.ac.uk

=over

=item get_pace_sequence

Returns a ref to a hash of information about clones either read
directly from pace or from a cache file.

=item get_pace_name

=item get_humace_sequence

