#
# Perl Module for humpub
#
# Cared for by Tim Hubbard <th@sanger.ac.uk>
#
# $Header: /nfs/disk100/humpub/modules.devel/RCS/humpubace.pm,v 1.8 1998/11/25 13:35:43 humpub Exp $
#
#Copyright Genome Research Limited (1997). Please see information on licensing in LICENSE

package humpubace;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Ace;
use Sequence;

=pod

=head1 humpubace.pm

Contains common functions needed for interatcion
with humace, and other acedb databases.

=cut

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw(getMethods
		makeWspecDir
		getDataDirectory
		makePasswdWrm
		initializeDatabase
		editDisplaysWrm
		timestamp
		getSequenceFromAce);

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

# write acefile of tag of an object
sub dump_acetag{
    my($clone,$acetag,$file,$host,$port,$rmess)=@_;
# read from db
    unless($host){
	die "host not defined";
    }
    my($db,$com1,$com2,$results);
    $com1="find sequence \"$clone\"";
    $com2="show -a -f $file $acetag";
    $db=Ace::AceDB->new($host,$port,100) ||
	die "Connection failure: ",Ace->error;
    $db->query($com1);
    if($db->status == STATUS_ERROR){
	print "$clone: Got an error ",$db->error();
	return 1;
    }
    $results='';
    while ($db->status == STATUS_PENDING) {
	$results .= $db->read;
    }
    if($results!~/Found 1 object/){
	$$rmess="$com1|$results";
	return 1;
    }
    $db->query($com2);
    if($db->status == STATUS_ERROR){
	print "$clone: Got an error ",$db->error();
	return 1;
    }
    $results='';
    while ($db->status == STATUS_PENDING) {
	$results .= $db->read;
    }
    if($results!~/1 object dumped/){
	$$rmess="$com1|$results";
	return 1;
    }
    return 0;
}

=pod

=head2 timestamp

    # Generate a timestamp string for now
    $timestamp = timestamp();

Generates an acedb timestamp (such as "1998-06-04_17:31:51_jgrg") for
inclusion in an ace file, taking a time string as input, or defaulting
to current time.

=cut

sub timestamp (;$) {
    my $time = shift || time;

    # Get time info
    my ($sec, $min, $hour, $mday, $mon, $year) = (localtime($time))[0..5];

    # Change numbers to double-digit format
    ($mon, $mday, $hour, $min, $sec) = ('00'..'59')[($mon + 1), $mday, $hour, $min, $sec];

    # Make year
    $year += 1900;

    # Translate the effective userID into a user name
    my $id = (getpwuid($>))[0];

    return "$year-$mon-${mday}_$hour:$min:${sec}_$id";
}

=pod

=head2 getMethods

    getMethods( $db, $fh );

Takes an B<AcePerl $db> database connection
object, and a B<FileHandle $fh> filehandle
object.  The acedb database is queried for all
it's methods, which are printed to B<$fh>.

=cut

sub getMethods {
    my( $db, $FH ) = @_;
    my $methods = $db->raw_query( "find Method *\n" );
    $methods = $db->raw_query( "show -a\n" );
    my $n = chr(0); # Had problem with this character in output
    $methods =~ s/$n//g;
    print $FH $methods;
}

=pod

=head2 makeWspecDir

    makeWspecDir( "homeDir", $db )

Uses aceserver's "wspec" command to create a wspec
directory in B<homeDir>, the home directory of
your acedb database.  B<$db> is an B<AcePerl>
database object.

=cut

sub makeWspecDir {
    my( $homeDir, $db ) = @_;
    my $wspec = "$homeDir/wspec";
    my ($wspecUU, @wspec, @wrmFiles);

    # Make wspec directory and cd into it
    mkdir $wspec, 0775; chdir( $wspec );

    # Get uuencoded tar file of wspec from server
    $wspecUU = $db->raw_query('wspec');
    @wspec = split /\n/, $wspecUU;

    # Open a pipe to tar to make wspec files,
    # and print UUdecoded stream to it.
    open WSPEC, "| tar xf - 2>/dev/null" or die;
    foreach (@wspec) {
	# UUdecodes @wspec
	last if /^end/;
	next if /^$/;
	next if /[a-z]/;
	next unless int((((ord() - 32) & 077) + 2) / 3) ==
	    int( length() / 4 );
	print WSPEC unpack "u", $_;
    }
    close WSPEC;
    
    # Change the mode of the files in wspec
    opendir WSPECDIR, $wspec or die;
    @wrmFiles = grep -f, readdir WSPECDIR;
    closedir WSPECDIR;
    chmod 0644,  @wrmFiles;  

    # Change working directory back to original
    chdir( $homeDir );
}

=pod

=head2 getDataDirectory

    $dbHome = getDataDirectory( $db );

Queries the server on the B<$db AcePerl>
connection for its home directory.

=cut

sub getDataDirectory {
    # Finds the home directory of the server
    my $db = $_[0];

    my $response = $db->raw_query('status');
    my ($dir) = $response =~ /Data directory (\S+)\//;
    return $dir;
}

=pod

=head2 makePasswdWrm

    makePasswdWrm( "homeDir", @users )

Makes a new wspec/passwd.wrm file, in the
directory B<homeDir>.  By default it adds the user in
$ENV{'USER'} as the sole entry.  If B<@users> is
supplied, then it adds all of them to the file.

=cut

sub makePasswdWrm {
    my( $homeDir, @users) = @_;
    my $passWrm = "${homeDir}/wspec/passwd.wrm";
    unless (@users) {
	unless ($users[0] = $ENV{'USER'}) {
	    die "USER environment variable not set: Can't make passwd.wrm\n";
	}
    }
    open PASSWD, ">$passWrm" or die;
    print PASSWD "// passwd.wrm generated by makePasswdWrm\n\n";
    print PASSWD map "$_\n\n", @users;
    close PASSWD; # Must close to ensure buffer is flushed into file
    chmod 0644, $passWrm;
}

=pod

=head2 initializeDatabase

    initializeDatabase( "homeDir", "tace" );

Initialises the database in the directory B<homeDir>
by starting up the program given by the B<tace>
argument, in the directory B<homeDir>, and answering
yes to the question "Should I initialise
database?".  It makes a "database" directory with
mode 0775, if it's not already present.

=cut

sub initializeDatabase {
    my( $homeDir, $tace ) = @_;
    my $dataDir = "${homeDir}/database";
    unless ( -e $dataDir ) {
	mkdir( $dataDir, 0775 );
    }
    system ("echo 'y\n' | $tace $homeDir >/dev/null") == 0 or die
	"Error: Can't initialise database\n";
}


=pod

=head2 editDisplaysWrm

    editDisplaysWrm( "homeDir", "title" );

Changes the title of the main acedb window of the
database in B<dbHome> to B<title>.

=cut

sub editDisplaysWrm {
    # Adds a title to displays.wrm for the database
    my( $homeDir, $title ) = @_;
    my $displays = "${homeDir}/wspec/displays.wrm";

    open DISP, "< $displays" or die;
    my @disp = <DISP>;
    close DISP;

    foreach (@disp) {
	next unless /^_DDtMain/;
	# Add our title onto the Main window
	s|\s-t\s+\S+| -t \"$title\"|i and last;
    }

    open DISPOUT, "> $displays" or die;
    print DISPOUT @disp;
    close DISPOUT;
}

=pod

=head2 getSequenceFromAce

    $dna = getSequenceFromAce( $seq );

C<$seq> is an B<Ace::Object> object of the acedb Sequence class.  It
returns a B<Sequence.pm> object, or undef on error.

=cut

sub getSequenceFromAce {
    my $seq = shift;
    return unless $seq->class() eq 'Sequence';

    my( $dnaTag, $DNA, $seqObj );
    my $name = $seq->name;

	# Is there a DNA_contig?         Get the DNA itself
    if ($dnaTag = $seq->at('DNA[1]') and ($DNA) = $dnaTag->fetch()->at()) {
	# Sequence must contain at least one character
	if (length($DNA)) {
	    $seqObj = Sequence->new_from_strings( $name, $DNA );
	    # Check that we got valid dna characters only
	    if ($seqObj->isdna_iub) {
		return $seqObj;
	    }
	}
    }
    return;
}

1;

__END__
